#! python3
import os
import numpy as numpy
#import Gnuplot as gp
import json
import pprint
import argparse

# folder hierarchy
# scenario: e.g. cloud x, resource y, ...
#TEST_DIRECTORY = "scalability_16_threads/" #+TEST_DATABASE+"/"+TEST_SETUP+"/"
#TODO: needs to be passed as cli argument
#SCENARIO_FOLDER = "blade/"

#TODO: needs to be passed as cli argument
#DBMS_CONFIG_FOLDER = "cassandra_persistTo_2/"

# data folder
DATA_FOLDER = "data/" 




def get_overall_data(case, dbms_result_folder, run):
    """
    Get overall test data from log files
    :param case: string e.g. load, run_workloadA, run_workloadB
    :param td: test deployment
    :return: array for all threads
    """
    runtime = []
    throughput = []

    #for run in RUN_FOLDERS:
    #    with open(SCENARIO_FOLDER+DBMS_CONFIG_FOLDER+dbms_result_folder+run+DATA_FOLDER+"load.txt", 'r') as file:
    if(case == "load"):
        file = open(dbms_result_folder+"/"+run+"/"+DATA_FOLDER+"load.txt", 'r')
    elif (case == "transaction"):
        file = open(dbms_result_folder+"/"+run+"/"+DATA_FOLDER+"transaction.txt", 'r')
    else:
        print("Unsupported result file type: " + case)
        exit()

    for line in file:
        line = line.rstrip()
        if "[OVERALL], RunTime" in line:
            runtime.append(
                (convert_string_to_float(line.split(', ')[-1]) / 1000 / 60)
                )
        if "[OVERALL], Throughput" in line:
            throughput.append(convert_string_to_float(line.split(', ')[-1]))

    data = {"runtime": runtime,
            "throughput": throughput}

    return data


def get_insert_data(case, dbms_result_folder, run):
    ins_op = []
    avg_lat = []

    #for runs in RUN_FOLDERS:
    #    with open(SCENARIO_FOLDER+DBMS_CONFIG_FOLDER+dbms_result_folder+runs+DATA_FOLDER+"load.txt", 'r') as file:
    if(case == "load"):
        file = open(dbms_result_folder+"/"+run+"/"+DATA_FOLDER+"load.txt", 'r')
    elif (case == "transaction"):
        file = open(dbms_result_folder+"/"+run+"/"+DATA_FOLDER+"transaction.txt", 'r')
    else:
        print("Unsupported result file type: " + case)
        exit()
    for line in file:
        line = line.rstrip()
        if "[INSERT], Operations" in line:
            ins_op.append(convert_string_to_float(line.split(', ')[-1]))
        if "[INSERT], AverageLatency" in line:
            avg_lat.append(convert_string_to_float(line.split(', ')[-1]))

    data = {"ins_op": ins_op,
            "ins_avg_lat": avg_lat}

    return data

#TODO: needs to be aligned! load.txt needs to be determined based on case!
def get_read_data(case, dbms_result_folder, run):
    read_op = []
    avg_lat = []

    #for runs in RUN_FOLDERS:
    #    with open(SCENARIO_FOLDER+DBMS_CONFIG_FOLDER+dbms_result_folder+runs+DATA_FOLDER+"load.txt", 'r') as file:
    if(case == "load"):
        file = open(dbms_result_folder+"/"+run+"/"+DATA_FOLDER+"load.txt", 'r')
    elif (case == "transaction"):
        file = open(dbms_result_folder+"/"+run+"/"+DATA_FOLDER+"transaction.txt", 'r')
    else:
        print("Unsupported result file type: " + case)
        exit()
    for line in file:
        line = line.rstrip()
        if "[READ], Operations" in line:
            read_op.append(convert_string_to_float(line.split(', ')[-1]))
        if "[READ], AverageLatency" in line:
            avg_lat.append(convert_string_to_float(line.split(', ')[-1]))

    data = {"read_op": read_op,
            "read_avg_lat": avg_lat}

    return data


def get_update_data(case, dbms_result_folder, run):
    upd_op = []
    avg_lat = []

    if(case == "load"):
        file = open(dbms_result_folder+"/"+run+"/"+DATA_FOLDER+"load.txt", 'r')
    elif (case == "transaction"):
        file = open(dbms_result_folder+"/"+run+"/"+DATA_FOLDER+"transaction.txt", 'r')
    else:
        print("Unsupported result file type: " + case)
        exit()
    for line in file:
        line = line.rstrip()
        if "[UPDATE], Operations" in line:
            upd_op.append(convert_string_to_float(line.split(', ')[-1]))
        if "[UPDATE], AverageLatency" in line:
            avg_lat.append(convert_string_to_float(line.split(', ')[-1]))

    data = {"upd_op": upd_op,
            "upd_avg_lat": avg_lat}

    return data


def get_scan_data(case, dbms_result_folder, run):
    upd_op = []
    avg_lat = []

    if(case == "load"):
        file = open(dbms_result_folder+"/"+run+"/"+DATA_FOLDER+"load.txt", 'r')
    elif (case == "transaction"):
        file = open(dbms_result_folder+"/"+run+"/"+DATA_FOLDER+"transaction.txt", 'r')
    else:
        print("Unsupported result file type: " + case)
        exit()
    for line in file:
        line = line.rstrip()
        if "[SCAN], Operations" in line:
            upd_op.append(convert_string_to_float(line.split(', ')[-1]))
        if "[SCAN], AverageLatency" in line:
            avg_lat.append(convert_string_to_float(line.split(', ')[-1]))

    data = {"scan_op": upd_op,
            "scan_avg_lat": avg_lat}

    return data



def convert_string_to_float(value):
    return round(float(value), 0)


def get_data():

    print("startextracting evaluation metrics...")
    parser = argparse.ArgumentParser()
    parser.add_argument('--resultFolder', '-rf',type=str,required=True, help="evaluation folder which contains the evaluation run results and the plot folder to store the extracted data, e.g.: omistack/read-heavy-blade/cassandra/node-3_replication-3_persistTo-TWO_readFrom-ONE")
    parser.add_argument('--workloadPhase', '-wp',type=str,required=True, help="workload phase: all, load, transaction")
    
    args = parser.parse_args()
    resultFolder=args.resultFolder
    metricPhase=args.workloadPhase


    #print("----------------------------")
    #print("Handling DBMS configuration: " + dbms_result_folder)
    runs = sorted(os.listdir(resultFolder))
        
    metricsOverAllRuns = {"metricsLoadPhase": {}, "metricsTransactionPhase": {}}
    #iterate over all runs
    for run in runs:
        print("----------------------------")
        print("Handling run folder: " + run)
        if run == "plots": 
            print("Found plots folder, skipping!")
            continue
            
            # load other data from methods
            #TODO: iterate over all load.txt files (not yet supported by the framework)
        metricsLoadPhase = {}    
        if metricPhase == "all" or metricPhase == "load":
            metricsLoadPhase.update(get_overall_data("load", resultFolder, run))    # returns metrics: throughput, runtime
            metricsLoadPhase.update(get_insert_data("load", resultFolder, run))     # returns metrics: ins_avg_lat, ins_op
            metricsLoadPhase.update(get_read_data("load", resultFolder, run))       # returns metrics: read_avg_lat, read_op
            metricsLoadPhase.update(get_update_data("load", resultFolder, run))     # returns metrics: upd_avg_lat, upd_op
            metricsLoadPhase.update(get_scan_data("load", resultFolder, run))     # returns metrics: scan_avg_lat, scan_op

        #TODO: iterate over all transaction files of the same type (not yet supported by the framework)
        metricsTransactionPhase = {}
        if metricPhase == "all" or metricPhase == "transaction":
            metricsTransactionPhase.update(get_overall_data("transaction", resultFolder, run))
            metricsTransactionPhase.update(get_insert_data("transaction", resultFolder, run))
            metricsTransactionPhase.update(get_read_data("transaction", resultFolder, run))
            metricsTransactionPhase.update(get_update_data("transaction", resultFolder, run))
            metricsTransactionPhase.update(get_scan_data("transaction", resultFolder, run))

        values = {"metricsLoadPhase": metricsLoadPhase,
                    "metricsTransactionPhase": metricsTransactionPhase}

        # write stats for each run
        #TODO: check for aggregation over multiple runs for transaction workloads! Currently not implemented!
        print("----------------------------")
        print("Writing run data for: " + run)
        with open(resultFolder+"/"+run+"/"+DATA_FOLDER+"data.json", 'w') as fp:
            json.dump(values, fp,indent=4)

        for metric in metricsLoadPhase:
            if metric in metricsOverAllRuns["metricsLoadPhase"]:
                metricsOverAllRuns["metricsLoadPhase"][metric] = numpy.append(metricsOverAllRuns["metricsLoadPhase"][metric], metricsLoadPhase[metric])
            else:
                    metricsOverAllRuns["metricsLoadPhase"][metric] = numpy.array(metricsLoadPhase[metric])
        for metric in metricsTransactionPhase:
            if metric in metricsOverAllRuns["metricsTransactionPhase"]:
                metricsOverAllRuns["metricsTransactionPhase"][metric] = numpy.append(metricsOverAllRuns["metricsTransactionPhase"][metric], metricsTransactionPhase[metric])
            else:
                metricsOverAllRuns["metricsTransactionPhase"][metric] = numpy.array(metricsTransactionPhase[metric])                     
                

    # calculate stats for DBMS configuration
    metricsForConfiguration = {"metricsLoadPhase": {}, "metricsTransactionPhase": {}}
    for metric in metricsOverAllRuns["metricsLoadPhase"]:
        if metricsOverAllRuns["metricsLoadPhase"][metric].size > 0:
            metricsForConfiguration["metricsLoadPhase"][metric] = {
                "avg": numpy.average(metricsOverAllRuns["metricsLoadPhase"][metric]),
                "std": numpy.std(metricsOverAllRuns["metricsLoadPhase"][metric]),
                "min": numpy.amin(metricsOverAllRuns["metricsLoadPhase"][metric]),
                "max": numpy.amax(metricsOverAllRuns["metricsLoadPhase"][metric])
            }
    for metric in metricsOverAllRuns["metricsTransactionPhase"]:
        if metricsOverAllRuns["metricsTransactionPhase"][metric].size > 0:
            metricsForConfiguration["metricsTransactionPhase"][metric] = {
                "avg": numpy.average(metricsOverAllRuns["metricsTransactionPhase"][metric]),
                "std": numpy.std(metricsOverAllRuns["metricsTransactionPhase"][metric]),
                "min": numpy.amin(metricsOverAllRuns["metricsTransactionPhase"][metric]),
                "max": numpy.amax(metricsOverAllRuns["metricsTransactionPhase"][metric])
            }

    # write stats for each DBMS configuration
    #TODO: check for aggregation over multiple transaction runs! Currently not implemented!
    print("----------------------------")
    print("Writing overall DBMS configuration data for: " + resultFolder)
    with open(resultFolder+"/plots/data.json", 'w') as fp:
        json.dump(metricsForConfiguration, fp,indent=4)
    #print(metricsForConfiguration)
        

        # create average for latency and other metrics
        #total_avg = generate_total_avg(aggr_values)
        #deployment_data.update({td: total_avg})

        # deprecated plot. but keep it at the moment in case of multiple runs with different threadcount
        #create_throughput_plot(total_avg, td, 'total')
        #create_latency_plot(total_avg, td, 'total')
        #create_runtime_plot(total_avg, td, 'total')
    print("----------------------------")
    print("Successfully extracted all data!")    

    # plot test data
    #print_runB_data(deployment_data, "throughput")
    #print_runB_data(deployment_data, "runtime")
    #print_runB_data(deployment_data, "upd_avg_lat")
    #print_runB_data(deployment_data, "read_avg_lat")


#init function call
get_data()
