#! python3
import os
import numpy as numpy
import json
import pprint
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import argparse



class TimeseriesData:
    timestamps = []
    operations = []
    latencies = []


def plot_timeseries():

    parser = argparse.ArgumentParser(description='Plot timeseries based on YCSB result files.')
    parser.add_argument('--input', '-i',type=str, nargs=1,required=True, help='path to the input YCSB file')
    parser.add_argument('--output', '-o',type=str, nargs=1,required=True, help='path to the store timeseries plot')
    args = parser.parse_args()

    inputPath = args.input[0]
    outputPath = args.output[0]

    #extract timeseries and store it in a list
    print("plotting throughput timeseries data from: " + inputPath)
    plot_throughput_timeseries(inputPath,outputPath)

    print("plotting latency timeseries data from: " + inputPath)
    plot_latency_timeseries(inputPath,outputPath)

    print("finished evaluation metrics plotting!")



def plot_throughput_timeseries(inputPath, outputPath):


    extractedTimeseriesData = TimeseriesData()

    file = open(inputPath, 'r', encoding='utf8')
    try:
        for line in file:
            line = line.rstrip()
            if "[TIMESERIES]" in line:
                #extract current ops per second
                splittedLine = line.split(";")
                operationItem = splittedLine[2]
                operationItem = operationItem.strip()
                #print (operationItem)
                extractedTimeseriesData.operations.append(float(operationItem))
                #extract timestamp
                secondsItem = splittedLine[1]
                secondsItem = secondsItem.strip()
                extractedTimeseriesData.timestamps.append(float(secondsItem))

    except UnicodeDecodeError: 
        print("Skipping line! Unable to decode line due to UnicodeDecodeError!")      

    
    print("plotting throughput timeseries...")
    #Plotting 
    
    plt.figure()
    plt.xlabel("Runtime in sec")
    plt.ylabel("Operations / second")
    plt.title("Throughput")
    
    plt.plot(extractedTimeseriesData.timestamps, extractedTimeseriesData.operations)    

    outputFile = outputPath + "-throughput-ts.png"
    print("saving throughput plot to disk: " + outputFile)
    plt.savefig(outputFile)   
   
    
    


def plot_latency_timeseries(inputPath, outputPath):

  
    extractedTimeseriesData = TimeseriesData()
    extractedTimeseriesData.timestamps = []


    file = open(inputPath, 'r', encoding='utf8')
    try:
        for line in file:
            line = line.rstrip()
            if "[TIMESERIES]" in line:
                #extract current ops per second
                splittedLine = line.split(";")
                latencyItem = splittedLine[3]
                latencyItem = latencyItem.strip() 
                #print (latencyItem)
                extractedTimeseriesData.latencies.append(float(latencyItem))
                #extract timestamp
                secondsItem = splittedLine[1]
                secondsItem = secondsItem.strip()
                extractedTimeseriesData.timestamps.append(float(secondsItem))

    except UnicodeDecodeError: 
        print("Skipping line! Unable to decode line due to UnicodeDecodeError!")       

    print("plotting latency timeseries...")
    #Plotting 
    plt.figure()
    plt.xlabel("Runtime in sec")
    plt.ylabel("Avg. Latency in us")
    plt.title("Latency")
    plt.plot(extractedTimeseriesData.timestamps, extractedTimeseriesData.latencies)    

    outputFile = outputPath + "-latency-ts.png"
    print("saving latency plot to disk: " + outputFile)
    plt.savefig(outputFile)
    
    



#init function call
plot_timeseries()