import argparse
#import matplotlib as plt
from influxdb import InfluxDBClient
import numpy as np
from datetime import datetime
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as pp
import os


class TimeseriesData:
    def __init__(self):
        # relative to evaluation run
        self.timestampsRelative = []
    # Unix timestamp
        self.timestampsAbsolute = []
        self.values = []

class CpuLoadPlot:
    def __init__(self, startTime, endTime, publicIpTag, influxDB):
        self.query = str('SELECT 100 - mean("usage_idle") AS "mean_cpu_load" FROM "' + influxDB + '"."autogen"."cpu" WHERE time > START_TIME_WILDCARD  AND time <= END_TIME_WILDCARD AND public_ip=\'PUBLIC_IP_WILDCARD\' AND "cpu"=\'cpu-total\' GROUP BY time(10s), "host" FILL(null)')
        self.query = self.query.replace('START_TIME_WILDCARD', str(startTime) + "s")
        self.query = self.query.replace("END_TIME_WILDCARD", str(endTime) +"s")
        self.query = self.query.replace("PUBLIC_IP_WILDCARD", str(publicIpTag))
       

        print(self.query)
       
        #self.datapoints = TimeseriesData()
        self.timestampsRelative = []
        self.timestampsAbsolute = []
        self.values = []
        self.xLabel = "runtime in s"
        self.yLabel = "Overall CPU Load in %"
        self.header = "CPU Load"   
        self.metricName = "mean_cpu_load"  

class MemoryPlot:
    def __init__(self, startTime, endTime, publicIpTag, influxDB):
        self.query = str('SELECT mean("used")/1000000000 AS "mean_used" FROM "' + influxDB + '"."autogen"."mem" WHERE time > START_TIME_WILDCARD AND time <= END_TIME_WILDCARD AND public_ip=\'PUBLIC_IP_WILDCARD\' GROUP BY time(10s) FILL(null)')
        self.query = self.query.replace('START_TIME_WILDCARD', str(startTime) + "s")
        self.query = self.query.replace("END_TIME_WILDCARD", str(endTime) +"s")
        self.query = self.query.replace("PUBLIC_IP_WILDCARD", str(publicIpTag))
       

        print(self.query)
       
        #self.datapoints = TimeseriesData()
        self.timestampsRelative = []
        self.timestampsAbsolute = []
        self.values = []
        self.xLabel = "runtime in s"
        self.yLabel = "Memory Usage in GB"
        self.header = "Memory Usage"   
        self.metricName = "mean_used"

class DiskWritesPlot:
    def __init__(self, startTime, endTime, publicIpTag, influxDB):
        self.query = str('SELECT non_negative_derivative(mean("writes"),10s) AS "disk_writes" FROM "' + influxDB + '"."autogen"."diskio" WHERE time > START_TIME_WILDCARD AND time <= END_TIME_WILDCARD AND public_ip=\'PUBLIC_IP_WILDCARD\' GROUP BY time(10s) FILL(null)')
        self.query = self.query.replace('START_TIME_WILDCARD', str(startTime) + "s")
        self.query = self.query.replace("END_TIME_WILDCARD", str(endTime) +"s")
        self.query = self.query.replace("PUBLIC_IP_WILDCARD", str(publicIpTag))
       

        print(self.query)
       
        #self.datapoints = TimeseriesData()
        self.timestampsRelative = []
        self.timestampsAbsolute = []
        self.values = []
        self.xLabel = "runtime in s"
        self.yLabel = "Disk I/O Writes"
        self.header = "Avg. Disk Writes per 10s "   
        self.metricName = "disk_writes"

class DiskReadsPlot:
    def __init__(self, startTime, endTime, publicIpTag, influxDB):
        self.query = str('SELECT non_negative_derivative(mean("reads"),10s) AS "disk_reads" FROM "' + influxDB + '"."autogen"."diskio" WHERE time > START_TIME_WILDCARD AND time <= END_TIME_WILDCARD AND public_ip=\'PUBLIC_IP_WILDCARD\' GROUP BY time(10s) FILL(null)')
        self.query = self.query.replace('START_TIME_WILDCARD', str(startTime) + "s")
        self.query = self.query.replace("END_TIME_WILDCARD", str(endTime) +"s")
        self.query = self.query.replace("PUBLIC_IP_WILDCARD", str(publicIpTag))
       

        print(self.query)
       
        #self.datapoints = TimeseriesData()
        self.timestampsRelative = []
        self.timestampsAbsolute = []
        self.values = []
        self.xLabel = "runtime in s"
        self.yLabel = "Disk I/O Reads"
        self.header = "Avg. Disk Reads per 10s "   
        self.metricName = "disk_reads"

class DiskIOUtilizationPlot:
    def __init__(self, startTime, endTime, publicIpTag, influxDB):
        self.query = str('SELECT non_negative_derivative(last("io_time"),1ms) AS "disk_io_utilization" FROM "' + influxDB + '"."autogen"."diskio" WHERE time > START_TIME_WILDCARD AND time <= END_TIME_WILDCARD AND public_ip=\'PUBLIC_IP_WILDCARD\'  GROUP BY time(10s) FILL(null)')
        self.query = self.query.replace('START_TIME_WILDCARD', str(startTime) + "s")
        self.query = self.query.replace("END_TIME_WILDCARD", str(endTime) +"s")
        self.query = self.query.replace("PUBLIC_IP_WILDCARD", str(publicIpTag))
       

        print(self.query)
       
        #self.datapoints = TimeseriesData()
        self.timestampsRelative = []
        self.timestampsAbsolute = []
        self.values = []
        self.xLabel = "runtime in s"
        self.yLabel = "Disk I/O Utilization"
        self.header = "Disk I/O Utilization per 10s "   
        self.metricName = "disk_io_utilization"

class NetworkPackagesSentPlot:
    def __init__(self, startTime, endTime, publicIpTag, influxDB):
        self.query = str('SELECT non_negative_derivative(mean("bytes_sent"),10s)/1000000 AS "bytes_sent_mb" FROM "' + influxDB + '"."autogen"."net" WHERE time > START_TIME_WILDCARD AND time <= END_TIME_WILDCARD AND public_ip=\'PUBLIC_IP_WILDCARD\' GROUP BY time(10s) FILL(null)')
        self.query = self.query.replace('START_TIME_WILDCARD', str(startTime) + "s")
        self.query = self.query.replace("END_TIME_WILDCARD", str(endTime) +"s")
        self.query = self.query.replace("PUBLIC_IP_WILDCARD", str(publicIpTag))
       

        print(self.query)
       
        #self.datapoints = TimeseriesData()
        self.timestampsRelative = []
        self.timestampsAbsolute = []
        self.values = []
        self.xLabel = "runtime in s"
        self.yLabel = "Bytes Sent in MB"
        self.header = "Avg. Bytes Sent"   
        self.metricName = "bytes_sent_mb"

class NetworkPackagesReceivedPlot:
    def __init__(self, startTime, endTime, publicIpTag, influxDB):
        self.query = str('SELECT non_negative_derivative(mean("bytes_recv"),10s)/1000000 AS "bytes_recv_mb" FROM "' + influxDB + '"."autogen"."net" WHERE time > START_TIME_WILDCARD AND time <= END_TIME_WILDCARD AND public_ip=\'PUBLIC_IP_WILDCARD\' GROUP BY time(10s) FILL(null)')
        self.query = self.query.replace('START_TIME_WILDCARD', str(startTime) + "s")
        self.query = self.query.replace("END_TIME_WILDCARD", str(endTime) +"s")
        self.query = self.query.replace("PUBLIC_IP_WILDCARD", str(publicIpTag))
       

        print(self.query)
       
        #self.datapoints = TimeseriesData()
        self.timestampsRelative = []
        self.timestampsAbsolute = []
        self.values = []
        self.xLabel = "runtime in s"
        self.yLabel = "Bytes Received in MB"
        self.header = "Avg. Bytes Received"   
        self.metricName = "bytes_recv_mb"        




def plotSystemMetrics():
    print("starting system metrics processing...")
    parser = argparse.ArgumentParser()
    parser.add_argument("--influxDB", default="telegraf", help="name of the InfluxDB, Default: telegraf")
    parser.add_argument("--influxUrl",required=True, help= "IP adress/DNS and port")
    parser.add_argument('--outputFolder', '-of',type=str,required=True, help="path to folder where png and raw result files will be saved")
    parser.add_argument("--startTime",type=int, required=True, help="start timestamp")
    parser.add_argument("--endTime",type=int ,required=True, help="end timestamp")
    parser.add_argument("--publicIpTag", type=str, required=True,help="public IP tag to select metric of specific VM")
    parser.add_argument("--nodeTypeTag", type=str, required=True,help="node type for plot name, e.g. dbms or workload-instance")
    parser.add_argument("--metric",required=True, choices=['cpu', 'mem', 'network', 'disk', 'all'], help="metric to be plotted")
    

    args = parser.parse_args()

    influxDB=args.influxDB
    influxUrl=args.influxUrl

    influxUrlParts = str(influxUrl).split(":")
    if(len(influxUrlParts) != 2):
        print("The provided influxUrl " + influxUrl +" is incorrect! Required format is: IP:PORT !!")
        quit()
    influxHost = influxUrlParts[0]
    influxPort = influxUrlParts[1]


    output=args.outputFolder
    publicIpTag=args.publicIpTag
    nodeType=args.nodeTypeTag
    startTime=args.startTime
    endTime=args.endTime
    metric=args.metric

    if(metric == 'cpu'):
        processCpuMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag, nodeType)
    elif(metric == 'mem'):
        processMemoryMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag, nodeType)
    elif(metric == 'net'):
        processNetworkSentMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag, nodeType)  
        processNetworkReceivedMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag, nodeType)
    elif(metric == 'disk'):
        processDiskWritesMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag,nodeType)  
        processDiskReadsMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag,nodeType)   
        processDiskIOUtilizationMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag,nodeType)   
    elif(metric == 'all'):
        processCpuMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag,nodeType)
        processMemoryMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag,nodeType)  
        processDiskWritesMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag,nodeType)
        processDiskReadsMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag,nodeType)  
        processDiskIOUtilizationMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag,nodeType)
        processNetworkSentMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag,nodeType)  
        processNetworkReceivedMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag,nodeType)
    else:
        print("(currently unsupported metric: " + metric)    


    print("finished system metrics processing!")
    
    #print(result)
    #print("Result: {0}".format(result))

def processCpuMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag, nodeTypeTag):
    print("processing CPU LOAD metrics...")
    # create a new instance of the InfluxDBClient
    client= InfluxDBClient(influxHost,influxPort, database=influxDB)

    # instantiate metric object
    metricObj = CpuLoadPlot(startTime, endTime, publicIpTag, influxDB)

    # get data rom InfluxDB
    result = client.query(metricObj.query)    
    dataPoints = list(result.get_points())
    
    counter = 0
    for dataPoint in dataPoints:
        #print(dataPoint)
        if(dataPoint[metricObj.metricName] != None):
            metricObj.timestampsRelative.append(float(counter))
            metricObj.timestampsAbsolute.append(dataPoint['time'])
            metricObj.values.append(float(dataPoint[metricObj.metricName]))
            counter = counter + 10

    plottingFile = os.path.join(str(output), 'cpuLoad-' + nodeTypeTag + '-' + publicIpTag + '.png')        
    dataFile = os.path.join(str(output), 'cpuLoad-' + nodeTypeTag + '-' + publicIpTag +  '.txt')

    
    saveMetricToDisk(metricObj.timestampsRelative, metricObj.timestampsAbsolute, metricObj.values, dataFile)
    plotTimeseries(metricObj.timestampsRelative, metricObj.values, metricObj.xLabel, metricObj.yLabel, metricObj.header, plottingFile)
    


def processMemoryMetric(influxHost, influxPort, influxDB, startTime, endTime, output, publicIpTag, nodeTypeTag):
    print("processing MEMORY USAGE  metrics...")
    # create a new instance of the InfluxDBClient
    client= InfluxDBClient(influxHost,influxPort, database=influxDB)

    # instantiate metric object
    metricObj = MemoryPlot(startTime, endTime, publicIpTag, influxDB)

    # get data rom InfluxDB
    result = client.query(metricObj.query)    
    dataPoints = list(result.get_points())
    
    counter = 0
    for dataPoint in dataPoints:
        #print(dataPoint)
        if(dataPoint[metricObj.metricName] != None):
            metricObj.timestampsRelative.append(float(counter))
            metricObj.timestampsAbsolute.append(dataPoint['time'])
            metricObj.values.append(float(dataPoint[metricObj.metricName]))
            counter = counter + 10

    plottingFile = os.path.join(str(output), 'memory-usage-' + nodeTypeTag + '-' + publicIpTag + '.png')           
    dataFile = os.path.join(str(output), 'memory-usage-node-' + nodeTypeTag + '-' + publicIpTag +  '.txt')

    
    saveMetricToDisk(metricObj.timestampsRelative, metricObj.timestampsAbsolute, metricObj.values, dataFile)
    plotTimeseries(metricObj.timestampsRelative, metricObj.values, metricObj.xLabel, metricObj.yLabel, metricObj.header, plottingFile)   


def processDiskWritesMetric(influxHost,influxPort,influxDB, startTime, endTime, output, publicIpTag, nodeTypeTag):
    print("processing DISK IO WRITES  metrics...")
    # create a new instance of the InfluxDBClient
    client= InfluxDBClient(influxHost,influxPort, database=influxDB)

    # instantiate metric object
    metricObj = DiskWritesPlot(startTime, endTime, publicIpTag, influxDB)

    # get data rom InfluxDB
    result = client.query(metricObj.query)    
    dataPoints = list(result.get_points())
    
    counter = 0
    for dataPoint in dataPoints:
        #print(dataPoint)
        if(dataPoint[metricObj.metricName] != None):
            metricObj.timestampsRelative.append(float(counter))
            metricObj.timestampsAbsolute.append(dataPoint['time'])
            metricObj.values.append(float(dataPoint[metricObj.metricName]))
            counter = counter + 10

    plottingFile = os.path.join(str(output), 'disk-io-writes-' + nodeTypeTag + '-' + publicIpTag + '.png')        
    dataFile = os.path.join(str(output), 'disk-io-writes-' + nodeTypeTag + '-' + publicIpTag +  '.txt')

    
    saveMetricToDisk(metricObj.timestampsRelative, metricObj.timestampsAbsolute, metricObj.values, dataFile)
    plotTimeseries(metricObj.timestampsRelative, metricObj.values, metricObj.xLabel, metricObj.yLabel, metricObj.header, plottingFile)

def processDiskReadsMetric(influxHost,influxPort,influxDB, startTime, endTime, output, publicIpTag, nodeTypeTag):
    print("processing DISK IO READS  metrics...")
    # create a new instance of the InfluxDBClient
    client= InfluxDBClient(influxHost,influxPort, database=influxDB)

    # instantiate metric object
    metricObj = DiskReadsPlot(startTime, endTime, publicIpTag, influxDB)

    # get data rom InfluxDB
    result = client.query(metricObj.query)    
    dataPoints = list(result.get_points())
    
    counter = 0
    for dataPoint in dataPoints:
        #print(dataPoint)
        if(dataPoint[metricObj.metricName] != None):
            metricObj.timestampsRelative.append(float(counter))
            metricObj.timestampsAbsolute.append(dataPoint['time'])
            metricObj.values.append(float(dataPoint[metricObj.metricName]))
            counter = counter + 10

    plottingFile = os.path.join(str(output), 'disk-io-reads-' + nodeTypeTag + '-' + publicIpTag + '.png')        
    dataFile = os.path.join(str(output), 'disk-io-reads-' + nodeTypeTag + '-' + publicIpTag +  '.txt')

    
    saveMetricToDisk(metricObj.timestampsRelative, metricObj.timestampsAbsolute, metricObj.values, dataFile)
    plotTimeseries(metricObj.timestampsRelative, metricObj.values, metricObj.xLabel, metricObj.yLabel, metricObj.header, plottingFile)

def processDiskIOUtilizationMetric(influxHost,influxPort,influxDB, startTime, endTime, output, publicIpTag, nodeTypeTag):
    print("processing DISK IO Utilization  metrics...")
    # create a new instance of the InfluxDBClient
    client= InfluxDBClient(influxHost,influxPort, database=influxDB)

    # instantiate metric object
    metricObj = DiskIOUtilizationPlot(startTime, endTime, publicIpTag, influxDB)

    # get data rom InfluxDB
    result = client.query(metricObj.query)    
    dataPoints = list(result.get_points())
    
    counter = 0
    for dataPoint in dataPoints:
        #print(dataPoint)
        if(dataPoint[metricObj.metricName] != None):
            metricObj.timestampsRelative.append(float(counter))
            metricObj.timestampsAbsolute.append(dataPoint['time'])
            metricObj.values.append(float(dataPoint[metricObj.metricName]))
            counter = counter + 10

    plottingFile = os.path.join(str(output), 'disk-io-utilization-' + nodeTypeTag + '-' + publicIpTag + '.png')        
    dataFile = os.path.join(str(output), 'disk-io-utilization-node-' + nodeTypeTag + '-' + publicIpTag +  '.txt')

    
    saveMetricToDisk(metricObj.timestampsRelative, metricObj.timestampsAbsolute, metricObj.values, dataFile)
    plotTimeseries(metricObj.timestampsRelative, metricObj.values, metricObj.xLabel, metricObj.yLabel, metricObj.header, plottingFile)    


def processNetworkSentMetric(influxHost,influxPort,influxDB, startTime, endTime, output, publicIpTag, nodeTypeTag):
    print("processing NETWORK SENT  metrics...")
    # create a new instance of the InfluxDBClient
    client= InfluxDBClient(influxHost,influxPort, database=influxDB)

    # instantiate metric object
    metricObj = NetworkPackagesSentPlot(startTime, endTime, publicIpTag, influxDB)

    # get data rom InfluxDB
    result = client.query(metricObj.query)    
    dataPoints = list(result.get_points())
    
    counter = 0
    for dataPoint in dataPoints:
        #print(dataPoint)
        if(dataPoint[metricObj.metricName] != None):
            metricObj.timestampsRelative.append(float(counter))
            metricObj.timestampsAbsolute.append(dataPoint['time'])
            metricObj.values.append(float(dataPoint[metricObj.metricName]))
            counter = counter + 10

    plottingFile = os.path.join(str(output), 'network-sent-' + nodeTypeTag + '-' + publicIpTag + '.png')        
    dataFile = os.path.join(str(output), 'network-sent-' + nodeTypeTag + '-' + publicIpTag +  '.txt')

    
    saveMetricToDisk(metricObj.timestampsRelative, metricObj.timestampsAbsolute, metricObj.values, dataFile)
    plotTimeseries(metricObj.timestampsRelative, metricObj.values, metricObj.xLabel, metricObj.yLabel, metricObj.header, plottingFile) 

def processNetworkReceivedMetric(influxHost,influxPort,influxDB, startTime, endTime, output, publicIpTag, nodeTypeTag):
    print("processing NETWORK RECEIVED  metrics...")
    # create a new instance of the InfluxDBClient
    client= InfluxDBClient(influxHost,influxPort, database=influxDB)

    # instantiate metric object
    metricObj = NetworkPackagesReceivedPlot(startTime, endTime, publicIpTag, influxDB)

    # get data rom InfluxDB
    result = client.query(metricObj.query)    
    dataPoints = list(result.get_points())
    
    counter = 0
    for dataPoint in dataPoints:
        #print(dataPoint)
        if(dataPoint[metricObj.metricName] != None):
            metricObj.timestampsRelative.append(float(counter))
            metricObj.timestampsAbsolute.append(dataPoint['time'])
            metricObj.values.append(float(dataPoint[metricObj.metricName]))
            counter = counter + 10

    plottingFile = os.path.join(str(output), 'network-received-' + nodeTypeTag + '-' + publicIpTag + '.png')        
    dataFile = os.path.join(str(output), 'network-received-node-' + nodeTypeTag + '-' + publicIpTag +  '.txt')

    
    saveMetricToDisk(metricObj.timestampsRelative, metricObj.timestampsAbsolute, metricObj.values, dataFile)
    plotTimeseries(metricObj.timestampsRelative, metricObj.values, metricObj.xLabel, metricObj.yLabel, metricObj.header, plottingFile)
    
    

def plotTimeseries(timestamps, values, xLabel, yLabel, header, output):
    print("Plotting values...")
    pp.figure()
    pp.xlabel(xLabel)
    pp.ylabel(yLabel)
    pp.title(header)
    pp.plot(timestamps, values)
    print("Saving plot to disk...")
    pp.savefig(output)

def saveMetricToDisk(relativeTimestamps, absoluteTimestamps, values, output):
    print("Saving values to disk...")
    numberOfItems = len(values)

    dataFile = open(output, 'w')
    dataFile.write("relativeTimestamp,absoluteTimestamp,value\n")

    for i in range(numberOfItems):
        dataFile.write(str(relativeTimestamps[i]) + "," + str(absoluteTimestamps[i]) + "," + str(values[i]) + "\n")

        

#init function call
plotSystemMetrics()





