#! python3
import os
import numpy as numpy
#import Gnuplot as gp
import json
import pprint
import argparse
import glob
from collections import OrderedDict
import math
import re
import sys


# round to the nearest ten
def roundup(x):
    return int(math.ceil(x / 10.0)) * 10

def saveTimeseriesToDisk(mergedResultsDic, output):
   
    dataFile = open(output, 'w')
    

    for keys,values in mergedResultsDic.items():
        dataFile.write('[TIMESERIES];' + str(keys) + ";"  + str(values) + "\n") 

def saveAggregatedDataToDisk(tag, key, value, output):
    
    dataFile = open(output, 'a')
    dataFile.write(str(tag) + "," + str(key) + ", "  + str(value) + "\n")                

   
def mergeTimeseries(results, resultFolder, outputFile):

    print('Start merging timeseries data...')
    mergedOperationDic = OrderedDict()
    mergedLatencyDic = OrderedDict()
    mergedResultsDic = OrderedDict()

    for resultFile in results:
        

        file = open(resultFile, 'r', encoding='utf8')
        try:
            for line in file:
                line = line.rstrip()
                #extractedTimeseriesData = TimeseriesData()
                if "current ops/sec" in line:
                    
                    splittedLine = line.split(";")

                    #extract timestamp
                    timestampPart = splittedLine[0]
                    timestampPart = timestampPart.split(" ")
                    yearItem = timestampPart[0].strip()
                    hourItem = timestampPart[1].strip()
                    secondsItem = timestampPart[2].strip()
                    secondsItem = roundup(int(secondsItem))
                    print(secondsItem)


                    #extract current ops per second
                    operationItem = splittedLine[1]
                    operationItem = operationItem.replace("current ops/sec","")
                    operationItem = operationItem.strip()

                    #extract avg. latency 
                    latencyItem = splittedLine[2]

                    #extract INSERT
                    latencyItem = re.search(r"\[INSERT:(.*?)\]", latencyItem) 
                    if(latencyItem is None):
                            print("Line does not contain any INSERT operations, checking now for READ operations")
                            latencyItem = splittedLine[2]
                            latencyItem = re.search(r"\[READ:(.*?)\]", latencyItem)
                    latencyItem = latencyItem.group(1)
                    print (latencyItem)
                    latencyItem = latencyItem.replace(" ","")
                    latencyItems = latencyItem.split(",")
                    avgItem = latencyItems[3]
                    avgItem = avgItem.replace("Avg=","")
                    print (avgItem)



                    try:
                        float(avgItem)
                    except ValueError:
                        print ("Current average latency value is not a number: " + avgItem + " Setting this item to 0 !")   
                        avgItem = 0 
                    
                    #TODO refactor whole script to use pandas library 
                    #extract READ
                    #extract UPDATE
                    #extract UPDATE
                    
                    
                  
                    if secondsItem in mergedOperationDic:
                        # seconds key is already there, add value
                        
                        sumUpOperations = float(mergedOperationDic[secondsItem]) +  float(operationItem)
                        mergedOperationDic[secondsItem] = sumUpOperations

                        mergedAvgItem = (float(mergedLatencyDic[secondsItem]) + float(avgItem))/2
                        mergedLatencyDic[secondsItem] = mergedAvgItem
                        
                    else:    
                        mergedOperationDic[secondsItem] = round(float(operationItem))
                        mergedLatencyDic[secondsItem] = float(avgItem)
                        
                    

        except UnicodeDecodeError: 
            print("Skipping line! Unable to decode line due to UnicodeDecodeError!")  

    #merge sum dict and latency dict into the mergedResult dic
    #check if dicts have the same size
    for keys,values in mergedOperationDic.items():
        mergedResultsDic[str(keys)] = str(values) + ";"

    for keys,values in mergedLatencyDic.items():
        mergedResultsDic[str(keys)] =mergedResultsDic[str(keys)] +  str(values) + ";"    

    
    saveTimeseriesToDisk(mergedResultsDic, resultFolder + outputFile)

def mergeOverallData(results, resultFolder, outputFile):

    print('Start merging aggregated result data...')

    mergedOverallDic = OrderedDict()
    mergedInsertDic = OrderedDict()
    mergedReadDic = OrderedDict()
    mergedUpdateDic = OrderedDict()

    for resultFile in results:
        

        file = open(resultFile, 'r', encoding='utf8')
        try:
            for line in file:
                line = line.rstrip()
                #extractedTimeseriesData = TimeseriesData()
                if "[OVERALL], RunTime(ms)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedOverallDic:
                        # key is already there, find take max value
                        if(int(mergedOverallDic[metricName])< int(valueItem)):
                            mergedOverallDic[metricName] = valueItem
                    else:    
                        mergedOverallDic[metricName] = int(valueItem)
                if "[OVERALL], Throughput(ops/sec)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedOverallDic:
                        # key is already there, sum up value
                        mergedOverallDic[metricName] = mergedOverallDic[metricName] + round(float(valueItem))
                    else:    
                        mergedOverallDic[metricName] = round(float(valueItem)) 
                ######################################### INSERT Operations        
                if "[INSERT], Operations" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedInsertDic:
                        # key is already there, sum up value
                        mergedInsertDic[metricName] = mergedInsertDic[metricName] + round(float(valueItem))
                    else:    
                        mergedInsertDic[metricName] = round(float(valueItem))
                if "[INSERT], AverageLatency(us)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedInsertDic:
                        # key is already there, sum up value
                        mergedInsertDic[metricName] = (mergedInsertDic[metricName] + float(valueItem))/2
                    else:    
                        mergedInsertDic[metricName] = float(valueItem)  
                if "[INSERT], MinLatency(us)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedInsertDic:
                        # key is already there,  take min value
                        if(int(mergedInsertDic[metricName])> int(valueItem)):
                            mergedInsertDic[metricName] = valueItem
                    else:    
                        mergedInsertDic[metricName] = int(valueItem) 
                if "[INSERT], MaxLatency(us)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedInsertDic:
                        # key is already there,  take min value
                        if(int(mergedInsertDic[metricName])< int(valueItem)):
                            mergedInsertDic[metricName] = valueItem
                    else:    
                        mergedInsertDic[metricName] = int(valueItem)
                if "[INSERT], Return=OK" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedInsertDic:
                        # key is already there, sum up value
                        mergedInsertDic[metricName] = mergedInsertDic[metricName] + int(valueItem)
                    else:    
                        mergedInsertDic[metricName] = int(valueItem)
                if "[INSERT], Return=ERROR" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedInsertDic:
                        # key is already there, sum up value
                        mergedInsertDic[metricName] = mergedInsertDic[metricName] + int(valueItem)
                    else:    
                        mergedInsertDic[metricName] = int(valueItem)          
                    ######################################### READ Operations
                if "[READ], Operations" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedReadDic:
                        # key is already there, sum up value
                        mergedReadDic[metricName] = mergedReadDic[metricName] + round(float(valueItem))
                    else:    
                        mergedReadDic[metricName] = round(float(valueItem))
                if "[READ], AverageLatency(us)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedReadDic:
                        # key is already there, sum up value
                        mergedReadDic[metricName] = (mergedReadDic[metricName] + float(valueItem))/2
                    else:    
                        mergedReadDic[metricName] = float(valueItem)  
                if "[READ], MinLatency(us)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedReadDic:
                        # key is already there,  take min value
                        if(int(mergedReadDic[metricName])> int(valueItem)):
                            mergedReadDic[metricName] = valueItem
                    else:    
                        mergedReadDic[metricName] = int(valueItem) 
                if "[READ], MaxLatency(us)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedReadDic:
                        # key is already there,  take min value
                        if(int(mergedReadDic[metricName])< int(valueItem)):
                            mergedReadDic[metricName] = valueItem
                    else:    
                        mergedReadDic[metricName] = int(valueItem)
                if "[READ], Return=OK" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedReadDic:
                        # key is already there, sum up value
                        mergedReadDic[metricName] = mergedReadDic[metricName] + int(valueItem)
                    else:    
                        mergedReadDic[metricName] = int(valueItem)
                if "[READ], Return=ERROR" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedReadDic:
                        # key is already there, sum up value
                        mergedReadDic[metricName] = mergedReadDic[metricName] + int(valueItem)
                    else:    
                        mergedReadDic[metricName] = int(valueItem)
                ########################################## Update Operations                                               
                if "[UPDATE], Operations" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedUpdateDic:
                        # key is already there, sum up value
                        mergedUpdateDic[metricName] = mergedUpdateDic[metricName] + round(float(valueItem))
                    else:    
                        mergedUpdateDic[metricName] = round(float(valueItem))
                if "[UPDATE], AverageLatency(us)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedUpdateDic:
                        # key is already there, sum up value
                        mergedUpdateDic[metricName] = (mergedUpdateDic[metricName] + float(valueItem))/2
                    else:    
                        mergedUpdateDic[metricName] = float(valueItem)  
                if "[UPDATE], MinLatency(us)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedUpdateDic:
                        # key is already there,  take min value
                        if(int(mergedUpdateDic[metricName])> int(valueItem)):
                            mergedUpdateDic[metricName] = valueItem
                    else:    
                        mergedUpdateDic[metricName] = int(valueItem) 
                if "[UPDATE], MaxLatency(us)" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedUpdateDic:
                        # key is already there,  take min value
                        if(int(mergedUpdateDic[metricName])< int(valueItem)):
                            mergedUpdateDic[metricName] = valueItem
                    else:    
                        mergedUpdateDic[metricName] = int(valueItem)
                if "[UPDATE], Return=OK" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedUpdateDic:
                        # key is already there, sum up value
                        mergedUpdateDic[metricName] = mergedUpdateDic[metricName] + int(valueItem)
                    else:    
                        mergedUpdateDic[metricName] = int(valueItem)
                if "[UPDATE], Return=ERROR" in line:
                    #extract current ops per second
                    splittedLine = line.split(",")
                    valueItem = splittedLine[2]
                    valueItem = valueItem.strip()
                    #extract timestamp
                    metricName = splittedLine[1]
                                      
                    if metricName in mergedUpdateDic:
                        # key is already there, sum up value
                        mergedUpdateDic[metricName] = mergedUpdateDic[metricName] + int(valueItem)
                    else:    
                        mergedUpdateDic[metricName] = int(valueItem)    

        except UnicodeDecodeError: 
            print("Skipping line! Unable to decode line due to UnicodeDecodeError!") 

    # write all overall data to result file
    print('Writing overall data to ' + resultFolder + outputFile)
    for keys,values in mergedOverallDic.items():
        saveAggregatedDataToDisk('[OVERALL]', keys, values, resultFolder + outputFile)

    # write insert data to result file
    print('Writing INSERT data to ' + resultFolder + outputFile)
    for keys,values in mergedInsertDic.items():
        saveAggregatedDataToDisk('[INSERT]', keys, values, resultFolder + outputFile)    

    # write insert data to result file
    print('Writing READ data to ' + resultFolder + outputFile)
    for keys,values in mergedReadDic.items():
        saveAggregatedDataToDisk('[READ]', keys, values, resultFolder + outputFile)     

    # write insert data to result file
    print('Writing Update data to ' + resultFolder + outputFile)
    for keys,values in mergedUpdateDic.items():
        saveAggregatedDataToDisk('[UPDATE]', keys, values, resultFolder + outputFile)            
            

def get_data():

    print("start merging evaluation metrics...")
    parser = argparse.ArgumentParser()
    parser.add_argument('--resultFolder', '-rf',type=str,required=True, help="result folder containing the load.txt and transaction.txt files")
    parser.add_argument('--workloadPhase', '-wp',type=str,required=True, help="workload phase: all, load, transaction")

    args = parser.parse_args()
    resultFolder=args.resultFolder
    workloadPhase=args.workloadPhase

    print("Calling plotting of workload phase: " + workloadPhase + " for result folder: " + resultFolder)

    if(workloadPhase=='load'):
        results = []
        for resultFile in glob.glob(resultFolder + '*_load.txt'):
            print('matching result file:' + resultFile)
            results.append(resultFile)
        mergeTimeseries(results,resultFolder,'load.txt') 
        mergeOverallData(results,resultFolder,'load.txt')    

    if(workloadPhase=='transaction'):
        results = []
        for resultFile in glob.glob(resultFolder + '*_transaction.txt'):
            print('matching result file:' + resultFile)
            results.append(resultFile)   
        mergeTimeseries(results,resultFolder,'transaction.txt')    
        mergeOverallData(results,resultFolder,'transaction.txt')    


    if(workloadPhase=='all'):
        results = []
        for resultFile in glob.glob(resultFolder + '*_load.txt'):
            print('matching result file:' + resultFile)
            results.append(resultFile)
        mergeTimeseries(results,resultFolder,'load.txt')  
        mergeOverallData(results,resultFolder,'load.txt')    

        results = []
        for resultFile in glob.glob(resultFolder + '*_transaction.txt'):
            print('matching result file:' + resultFile)
            results.append(resultFile)   
        mergeTimeseries(results,resultFolder,'transaction.txt')     
        mergeOverallData(results,resultFolder,'transaction.txt')         


       


#init function call
get_data()