package de.uulm.omi.dbms.evlautor.server.api;

import de.uulm.omi.cloudiator.dbms.evaluator.model.*;
import de.uulm.omi.dbms.evlautor.server.api.TemplateApiService;
import de.uulm.omi.dbms.evlautor.server.api.factories.TemplateApiServiceFactory;

import io.swagger.annotations.ApiParam;
import io.swagger.jaxrs.*;

import de.uulm.omi.cloudiator.dbms.evaluator.model.Cloud;
import de.uulm.omi.cloudiator.dbms.evaluator.model.CloudConfiguration;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Error;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Hardware;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Image;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Location;
import de.uulm.omi.cloudiator.dbms.evaluator.model.VM;

import java.util.Map;
import java.util.List;
import de.uulm.omi.dbms.evlautor.server.api.NotFoundException;

import java.io.InputStream;

import org.glassfish.jersey.media.multipart.FormDataContentDisposition;
import org.glassfish.jersey.media.multipart.FormDataParam;

import javax.servlet.ServletConfig;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import javax.ws.rs.*;
import javax.validation.constraints.*;

@Path("/template")

@Produces({ "application/json" })
@io.swagger.annotations.Api(description = "the template API")
@javax.annotation.Generated(value = "io.swagger.codegen.languages.JavaJerseyServerCodegen", date = "2019-07-16T22:13:00.743+02:00")
public class TemplateApi  {
   private final TemplateApiService delegate;

   public TemplateApi(@Context ServletConfig servletContext) {
      TemplateApiService delegate = null;

      if (servletContext != null) {
         String implClass = servletContext.getInitParameter("TemplateApi.implementation");
         if (implClass != null && !"".equals(implClass.trim())) {
            try {
               delegate = (TemplateApiService) Class.forName(implClass).newInstance();
            } catch (Exception e) {
               throw new RuntimeException(e);
            }
         } 
      }

      if (delegate == null) {
         delegate = TemplateApiServiceFactory.getTemplateApi();
      }

      this.delegate = delegate;
   }

    @GET
    @Path("/cloud")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "Get supported clouds", notes = "Returns a human readable list of supported clouds to run evaluations in the format id:name   ", response = Cloud.class, responseContainer = "List", tags={ "management", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the list of registered clouds", response = Cloud.class, responseContainer = "List"),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response templateCloudGet(@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.templateCloudGet(securityContext);
    }
    @POST
    @Path("/cloud")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "Register cloud", notes = "Register a new cloud in Mowgli to run evaluation scenarios", response = Cloud.class, tags={ "management", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the registered cloud", response = Cloud.class),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response templateCloudPost(@ApiParam(value = "the cloud api type",required=true, allowableValues="OPENSTACK, EC2") @QueryParam("api") String api
,@ApiParam(value = "the cloud api version, EC=default,  OpenStack = V2 or V3",required=true, allowableValues="DEFAULT, V2, V3") @QueryParam("apiVersion") String apiVersion
,@ApiParam(value = "the asociated user name  for the cloud, for EC2 the accesskeyId, | for OpenStack V2  projekt:user, for OpenStack V3 default:projekt_uuid:user",required=true) @QueryParam("user") String user
,@ApiParam(value = "the associated apikey (EC2) or password (OpenStack) for the cloud user",required=true) @QueryParam("secret") String secret
,@ApiParam(value = "a list of cloud speicifc configuration options (optional)" ,required=true) CloudConfiguration cloudConfiguration
,@ApiParam(value = "the cloud api endpoint URL, only required for OpenStack") @QueryParam("apiEndpoint") String apiEndpoint
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.templateCloudPost(api,apiVersion,user,secret,cloudConfiguration,apiEndpoint,securityContext);
    }
    @GET
    @Path("/hardware")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "Get supported hardware", notes = "Returns a human readable list of supported hardware flavours to run evaluations  ", response = Hardware.class, responseContainer = "List", tags={ "management", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the list of supported Hardware flavours", response = Hardware.class, responseContainer = "List"),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response templateHardwareGet(@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.templateHardwareGet(securityContext);
    }
    @GET
    @Path("/image")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "Get supported images", notes = "Returns a human readable list of supported images to run evaluations ", response = Image.class, responseContainer = "List", tags={ "management", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the list of supported images", response = Image.class, responseContainer = "List"),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response templateImageGet(@ApiParam(value = "the name of image distribution, will be used to search in the  cloud image names",required=true, allowableValues="UBUNTU") @QueryParam("imageDistribution") String imageDistribution
,@ApiParam(value = "the version of image distribution, will be used to search in the cloud image names",required=true, allowableValues="16") @QueryParam("imageVersion") String imageVersion
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.templateImageGet(imageDistribution,imageVersion,securityContext);
    }
    @GET
    @Path("/location")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "Get supported locations", notes = "Returns a human readable list of supported locations to run evaluations  ", response = Location.class, responseContainer = "List", tags={ "management", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns the list of supported locations", response = Location.class, responseContainer = "List"),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response templateLocationGet(@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.templateLocationGet(securityContext);
    }
    @GET
    @Path("/vm")
    
    @Produces({ "application/json" })
    @io.swagger.annotations.ApiOperation(value = "Get VM templates", notes = "Returns a VM template based on the provided constraints, for RAM use MB values, e.g. 4096  ", response = VM.class, responseContainer = "List", tags={ "management", })
    @io.swagger.annotations.ApiResponses(value = { 
        @io.swagger.annotations.ApiResponse(code = 200, message = "returns a list of matching vmtemplates", response = VM.class, responseContainer = "List"),
        
        @io.swagger.annotations.ApiResponse(code = 200, message = "Unexpected error", response = Error.class) })
    public Response templateVmGet(@ApiParam(value = "the id of a registered cloud",required=true) @QueryParam("cloud") Long cloud
,@ApiParam(value = "the name of a valid availability zone",required=true) @QueryParam("location") String location
,@ApiParam(value = "the name of image distribution, will be used to search in the  cloud image names",required=true, allowableValues="UBUNTU") @QueryParam("imageDistribution") String imageDistribution
,@ApiParam(value = "the version of image distribution, will be used to search in the cloud image names",required=true, allowableValues="16") @QueryParam("imageVersion") String imageVersion
,@ApiParam(value = "number of cores, will be used to search in the cloud hardware offers",required=true) @QueryParam("cores") Integer cores
,@ApiParam(value = "memory in MB, will be used to search in the cloud hardware offers",required=true) @QueryParam("memory") Integer memory
,@ApiParam(value = "disk size GB, will be used to search in the cloud hardware offers") @QueryParam("disk") Integer disk
,@Context SecurityContext securityContext)
    throws NotFoundException {
        return delegate.templateVmGet(cloud,location,imageDistribution,imageVersion,cores,memory,disk,securityContext);
    }
}
