package de.uulm.omi.dbms.evlautor.server.api.impl;

import de.uniulm.omi.cloudiator.colosseum.client.Client;
import de.uniulm.omi.cloudiator.colosseum.client.entities.Api;
import de.uniulm.omi.cloudiator.colosseum.client.entities.Cloud;
import de.uniulm.omi.cloudiator.colosseum.client.entities.CloudCredential;
import de.uniulm.omi.cloudiator.colosseum.client.entities.CloudProperty;
import de.uniulm.omi.cloudiator.colosseum.client.entities.HardwareOffer;
import de.uniulm.omi.cloudiator.colosseum.client.entities.Image;
import de.uniulm.omi.cloudiator.colosseum.client.entities.Location;
import de.uulm.omi.cloudiator.dbms.evaluator.model.CloudConfiguration;
import de.uulm.omi.cloudiator.dbms.evaluator.model.CloudConfigurationItem;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Error;
import de.uulm.omi.cloudiator.dbms.evaluator.model.VM;
import de.uulm.omi.dbms.evlautor.server.api.NotFoundException;
import de.uulm.omi.dbms.evlautor.server.api.TemplateApiService;
import de.uulm.omi.utils.ColosseumClientHelper;
import de.uulm.omi.utils.TemplateHelper;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.NoSuchElementException;
import javax.validation.constraints.NotNull;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@javax.annotation.Generated(value = "io.swagger.codegen.languages.JavaJerseyServerCodegen", date = "2018-11-12T16:16:38.936Z")
public class TemplateApiServiceImpl extends TemplateApiService {

    private static final Logger LOGGER = LoggerFactory.getLogger(TemplateApiServiceImpl.class);

    private static final String CLOUD_API_EC2 = "EC2";
    private static final String CLOUD_API_OPENSTACK = "OPENSTACK";

    private static final String CLOUD_API_VERSION_DEFAULT = "DEFAULT";
    private static final String CLOUD_API_VERSION_2 = "V2";
    private static final String CLOUD_API_VERSION_3 = "V3";

    //Cloudiator Network Settings
    private static final String SWORD_NETWORK_KEY_EC2 = "sword.ec2.default.vpc";
    private static final String SWORD_NETWORK_KEY_OPENSTACK_V2 = "sword.openstack.defaultNetwork";
    private static final String SWORD_NETWORK_KEY_OPENSTACK_V3 = "sword.openstack4j.defaultNetwork";

    //Cloudiator EC2 Image Settings
    private static final String SWORD_AMI_QUERY_KEY_EC2 = "sword.ec2.ami.query";
    private static final String SWORD_AMI_CC_QUERY_KEY_EC2 = "sword.ec2.ami.cc.query";


    @Override
    public Response templateCloudGet(SecurityContext securityContext) throws NotFoundException {

        Client cloudsClient = ColosseumClientHelper.getColosseumClient();

        List<Cloud> clouds = cloudsClient.controller(Cloud.class).getList();

        List<de.uulm.omi.cloudiator.dbms.evaluator.model.Cloud> mowgliClouds = new ArrayList<>();

        for (Cloud cloud: clouds) {
            de.uulm.omi.cloudiator.dbms.evaluator.model.Cloud mowgliCloud = new de.uulm.omi.cloudiator.dbms.evaluator.model.Cloud();
            mowgliCloud.setName(cloud.getName());
            mowgliCloud.setId(cloud.getId());
            mowgliClouds.add(mowgliCloud);

        }

        return Response.ok().entity(mowgliClouds).build();
    }



    /*
    @Override
    public Response templateCloudUpdateConfigurationPost(@NotNull Long idCloud,
        CloudConfiguration cloudConfiguration, SecurityContext securityContext)
        throws NotFoundException {

        Client upateCloudConfigurationClient = ColosseumClientHelper.getColosseumClient();

        Cloud cloud = upateCloudConfigurationClient.controller(Cloud.class).get(idCloud);
        Api api = upateCloudConfigurationClient.controller(Api.class).get(cloud.getApi());

        //delete all CloudProeprties for cloud before adding new properties
        List<CloudProperty> cloudProperties = upateCloudConfigurationClient
            .controller(CloudProperty.class).getList();
        for (CloudProperty cloudProperty: cloudProperties) {

            if(cloudProperty.getCloud().equals(idCloud)){
                upateCloudConfigurationClient.controller(CloudProperty.class).delete(cloudProperty);
                LOGGER.debug("Deleted existing CloudProperty " + cloudProperty.getId() + " before inserting new CloudProperties!");
            }

        }

        //####################### cloud configurations
        for (CloudConfigurationItem cloudConfigurationItem: cloudConfiguration) {

            String swordKey = null;
            String swordValue = null;
            CloudProperty cloudProperty = null;
            switch (cloudConfigurationItem.getConfigKey()){
                case NETWORK:

                    if(api.getName().contains("ec2")){
                        swordKey = SWORD_NETWORK_KEY_EC2;

                    }else if(api.getName().contains("openstack")){
                        //TODO: check also for V2
                        swordKey = SWORD_NETWORK_KEY_OPENSTACK_V3;

                    }else {
                        LOGGER.error("Unsupported cloud API: " + api);
                        return Response.serverError().entity("Unsupported cloud API: " + api + "! Only EC2 and OPENSTACK are supported!").build();
                    }
                    swordValue = cloudConfigurationItem.getConfigValue();
                    break;


                case AMICCQUERY:
                    swordKey = SWORD_AMI_CC_QUERY_KEY_EC2;
                    swordValue = "image-id=" + cloudConfigurationItem.getConfigValue();
                    break;

                case AMIQUERY:
                    swordKey = SWORD_AMI_QUERY_KEY_EC2;
                    swordValue = "image-id=" + cloudConfigurationItem.getConfigValue();
                    break;

                default:
                    LOGGER.error("Unsupported cloud configuration item key: " + cloudConfigurationItem.getConfigKey().toString());


            }
            if (swordKey == null || swordValue == null) {
                LOGGER.error("Wrong cloud configration! Skippng cloud configuration item!");
                return Response.serverError().entity("Wrong cloud configration! Skippng cloud configuration item!").build();
            }
            cloudProperty = new CloudProperty(swordKey,swordValue,idCloud);
            upateCloudConfigurationClient.controller(CloudProperty.class).create(cloudProperty);
            LOGGER.debug("Created cloud property with key: " + swordKey + " and value: " + swordValue);


        }
        return null;
    }

    */

    @Override
    public Response templateCloudPost(@NotNull String api, @NotNull String apiVersion,
        @NotNull String user, @NotNull String secret, CloudConfiguration cloudConfiguration,
        String apiEndpoint, SecurityContext securityContext) throws NotFoundException {

        Client registerCloudClient = ColosseumClientHelper.getColosseumClient();

        Api createdApi = null;
        String cloudName = "defaultCloud";
        //#################################### create API
        String pattern = "dd-MM-yyyy_HH:mm:ss";
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);
        String creationTime = simpleDateFormat.format(new Date());
        if(api.equals(CLOUD_API_EC2)){

            if(apiVersion.equals(CLOUD_API_VERSION_DEFAULT)){

                LOGGER.debug("Creating API entry for EC2 in Cloudiator!");
                Api ec2Api = new Api("ec2-api-"+creationTime,"aws-ec2");
                createdApi = registerCloudClient.controller(Api.class).create(ec2Api);
                cloudName = "ec2";


            }else {
                LOGGER.error("Unsupported cloud API version: " + apiVersion);
                return Response.serverError().entity("Unsupported cloud API version: " + apiVersion + "! For EC2 only DEFAULT is supported!").build();
            }

        }else if(api.equals(CLOUD_API_OPENSTACK)){

            cloudName = "openstack";

            if(apiVersion.equals(CLOUD_API_VERSION_2)){

                LOGGER.debug("Creating API entry for OPENSTACK V2 in Cloudiator!");
                Api openstackV2Api = new Api("openstackV2-api-"+creationTime, "openstack-nova");
                createdApi = registerCloudClient.controller(Api.class).create(openstackV2Api);


            }else if(apiVersion.equals(CLOUD_API_VERSION_3)){
                LOGGER.debug("Creating API entry for OPENSTACK V3 in Cloudiator!");
                Api openstackV3Api = new Api("openstackV3-api-"+creationTime,"openstack4j");
                createdApi = registerCloudClient.controller(Api.class).create(openstackV3Api);

            }else {
                LOGGER.error("Unsupported cloud API version: " + apiVersion);
                return Response.serverError().entity("Unsupported cloud API version: " + apiVersion + "! For OPENSTACK only V2 or V3 are supported!").build();
            }


        }else {
            LOGGER.error("Unsupported cloud API: " + api);
            return Response.serverError().entity("Unsupported cloud API: " + api + "! Only EC2 and OPENSTACK are supported!").build();
        }

        try {
            Thread.sleep(3000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        //#################################### create Cloud
        Cloud evaluationCloud = new Cloud(cloudName+"-"+creationTime, apiEndpoint, createdApi.getId());
        Cloud createdCloud = registerCloudClient.controller(Cloud.class).create(evaluationCloud);
        try {
            Thread.sleep(3000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        //#################################### create CloudCredential
        CloudCredential evaluationCredential = new CloudCredential(user, secret, createdCloud.getId(),
            (long) 1);
        registerCloudClient.controller(CloudCredential.class).create(evaluationCredential);

        de.uulm.omi.cloudiator.dbms.evaluator.model.Cloud mowgliCloud = new de.uulm.omi.cloudiator.dbms.evaluator.model.Cloud();

        mowgliCloud.setId(createdCloud.getId());
        mowgliCloud.setName(evaluationCloud.getName());

        //####################### cloud configurations
        for (CloudConfigurationItem cloudConfigurationItem: cloudConfiguration) {

            String swordKey = null;
            String swordValue = null;
            CloudProperty cloudProperty = null;
            switch (cloudConfigurationItem.getConfigKey()){
                case NETWORK:

                    if(api.equals(CLOUD_API_EC2)){
                        swordKey = SWORD_NETWORK_KEY_EC2;

                    }else if(api.equals(CLOUD_API_OPENSTACK)){

                        if(apiVersion.equals(CLOUD_API_VERSION_2)){
                            swordKey = SWORD_NETWORK_KEY_OPENSTACK_V2;
                        }else if(apiVersion.equals(CLOUD_API_VERSION_3)){
                            swordKey = SWORD_NETWORK_KEY_OPENSTACK_V3;
                        }

                    }else {
                        LOGGER.error("Unsupported cloud API: " + api + " for deriving network key!");
                        return Response.serverError().entity("Unsupported cloud API: " + api + "! Only EC2 and OPENSTACK are supported!").build();
                    }
                    swordValue = cloudConfigurationItem.getConfigValue();
                    break;


                case AMICCQUERY:
                    swordKey = SWORD_AMI_CC_QUERY_KEY_EC2;
                    swordValue = "image-id=" + cloudConfigurationItem.getConfigValue();
                    break;

                case AMIQUERY:
                    swordKey = SWORD_AMI_QUERY_KEY_EC2;
                    swordValue = "image-id=" + cloudConfigurationItem.getConfigValue();
                    break;

                 default:
                     LOGGER.error("Unsupported cloud configuration item key: " + cloudConfigurationItem.getConfigKey().toString());


            }
            if (swordKey == null || swordValue == null) {
                LOGGER.error("Wrong cloud configration! Skippng cloud configuration item!");
                return Response.serverError().entity("Wrong cloud configration! Skippng cloud configuration item!").build();
            }
            cloudProperty = new CloudProperty(swordKey,swordValue,createdCloud.getId());
            registerCloudClient.controller(CloudProperty.class).create(cloudProperty);
            LOGGER.debug("Created cloud property with key: " + swordKey + " and value: " + swordValue);


        }


        return Response.ok().entity(mowgliCloud).build();
    }


    @Override
    public Response templateHardwareGet(SecurityContext securityContext) throws NotFoundException {
        // do some magic!

        Client hardwareOfferClient = ColosseumClientHelper.getColosseumClient();

        List<HardwareOffer> hardwareOffers = hardwareOfferClient.controller(HardwareOffer.class).getList();

        List<de.uulm.omi.cloudiator.dbms.evaluator.model.Hardware> mowgliHardwares = new ArrayList<>();

        for (HardwareOffer hardwareOffer : hardwareOffers) {

            de.uulm.omi.cloudiator.dbms.evaluator.model.Hardware mowgliHardware = new de.uulm.omi.cloudiator.dbms.evaluator.model.Hardware();
            mowgliHardware.setCores(Long.valueOf(hardwareOffer.getNumberOfCores()));
            mowgliHardware.setMemory(hardwareOffer.getMbOfRam());
            mowgliHardware.setDisk(hardwareOffer.getLocalDiskSpace());
            //mowgliImage.setId(image.getId());
            mowgliHardwares.add(mowgliHardware);
        }

        return Response.ok().entity(mowgliHardwares).build();

    }


  @Override
  public Response templateImageGet(String imageDistribution, String imageVersion, SecurityContext securityContext) throws NotFoundException {

        Client imageClient = ColosseumClientHelper.getColosseumClient();

        List<de.uulm.omi.cloudiator.dbms.evaluator.model.Image> mowgliImages = new ArrayList<>();

        /*
        List<Image> images = imageClient.controller(Image.class)
            .getList(p -> (p.getName().toLowerCase().matches(".*" + "ubuntu" +  ".*") && p.getName().toLowerCase().matches(".*"+ "16.04" +".*")));

        List<de.uulm.omi.cloudiator.dbms.evaluator.model.Image> mowgliImages = new ArrayList<>();

        for (Image image: images) {

            de.uulm.omi.cloudiator.dbms.evaluator.model.Image mowgliImage = new de.uulm.omi.cloudiator.dbms.evaluator.model.Image();
            mowgliImage.setName(image.getName());
            mowgliImage.setId(image.getId());

            mowgliImages.add(mowgliImage);

        }

         */
        List<Image> images = imageClient.controller(Image.class).getList();

        for (Image image : images) {

            if(image.getName().toLowerCase().matches(".*" + "ubuntu" +  ".*")
                && image.getName().toLowerCase().matches(".*"+ "16" +".*")){

              de.uulm.omi.cloudiator.dbms.evaluator.model.Image mowgliImage = new de.uulm.omi.cloudiator.dbms.evaluator.model.Image();
              mowgliImage.setName(image.getName());
              mowgliImage.setId(image.getId());

              mowgliImages.add(mowgliImage);
            }

        }

        return Response.ok().entity(mowgliImages).build();


        //TODO: use the following snippet to query available images
        //imageClient.controller(Image.class)...

        //TODO: get all images from Colosseum via the image client

        //TODO: select all images which have ubuntu in its name

        //TODO fill in the following list with the matching images as string IMAGE_ID:IMAGE_NAME, e.g. 1:ubuntu1604



    }
    @Override
    public Response templateLocationGet(SecurityContext securityContext) throws NotFoundException {


        Client client = ColosseumClientHelper.getColosseumClient();

        List<Location> locations = client.controller(Location.class)
            .getList(p -> p.getIsAssignable());

        List<de.uulm.omi.cloudiator.dbms.evaluator.model.Location> mowgliLocations = new ArrayList<>();

        for (Location location: locations) {

            de.uulm.omi.cloudiator.dbms.evaluator.model.Location mowgliLocation = new de.uulm.omi.cloudiator.dbms.evaluator.model.Location();
            mowgliLocation.setName(location.getName());
            mowgliLocation.setId(location.getId());
            mowgliLocations.add(mowgliLocation);

        }

        return Response.ok().entity(mowgliLocations).build();
    }

    @Override
    public Response templateVmGet(@NotNull Long idCloud, @NotNull String location,
        @NotNull String imageDistribution, @NotNull String imageVersion, @NotNull Integer cores,
        @NotNull Integer memory, Integer disk, SecurityContext securityContext) throws NotFoundException {


        List<VM> vmTemplates = new ArrayList<>();


        try{
            vmTemplates = TemplateHelper
                .buildVMTemplates(idCloud,location,imageDistribution,imageVersion, cores, memory, disk);
            return Response.ok().entity(vmTemplates).build();
        }catch (NoSuchElementException e){

            LOGGER.debug("caought no such element execption!" + e.getMessage());

            Error notFoundError = new Error();
            notFoundError.setMessage(e.getMessage());
            notFoundError.setCode(404);

            return Response.serverError().entity(notFoundError).build();

        }



    }


}
