package de.uulm.omi.evaluation.jungle;


import de.uniulm.omi.cloudiator.colosseum.client.Client;
import de.uniulm.omi.cloudiator.colosseum.client.entities.ApplicationComponent;
import de.uniulm.omi.cloudiator.colosseum.client.entities.Instance;
import de.uniulm.omi.cloudiator.colosseum.client.entities.LifecycleComponent;
import de.uniulm.omi.cloudiator.colosseum.client.entities.VirtualMachine;
import de.uulm.omi.cloudiator.dbms.evaluator.model.FailureSpec.FailureLevelEnum;
import de.uulm.omi.cloudiator.dbms.evaluator.model.FailureSpec.NodeTypeEnum;
import de.uulm.omi.cloudiator.dbms.evaluator.model.GibbonSpec;
import de.uulm.omi.utils.ColosseumClientHelper;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Daniel Seybold on 07.02.2018.
 */
public class GibbonAlgorithm {

  private  static final Logger LOGGER = LoggerFactory.getLogger(GibbonAlgorithm.class);



  public static boolean injectFailure(Long applicationInstanceId, GibbonSpec gibbonSpec){

    boolean result = false;

    if(gibbonSpec.getFailureSpec().getFailureLevel().equals(FailureLevelEnum.CLOUD)){
      LOGGER.warn("CLOUD failure injection currently not implemented!");
    }
    else if(gibbonSpec.getFailureSpec().getFailureLevel().equals(FailureLevelEnum.REGION)){
      LOGGER.warn("REGION failure injection currently not implemented!");
    }
    else if(gibbonSpec.getFailureSpec().getFailureLevel().equals(FailureLevelEnum.ZONE)){
      LOGGER.warn("ZONE failure injection currently not implemented!");
    }
    else if(gibbonSpec.getFailureSpec().getFailureLevel().equals(FailureLevelEnum.VM)){
      result = injectVmFailure(applicationInstanceId, gibbonSpec.getFailureSpec().getNodeType());
    }
    else if(gibbonSpec.getFailureSpec().getFailureLevel().equals(FailureLevelEnum.SERVER)){
      LOGGER.warn("SERVER failure injection currently not implemented!");
    }

    return result;


  }

  private static boolean injectVmFailure(Long applicationInstanceId, NodeTypeEnum nodeType){

    Client gibbonClient = ColosseumClientHelper.getColosseumClient();


    List<Instance> instances = gibbonClient.controller(Instance.class).getList();
    //TODO: extend failure spec to search for a specific VM?
    //TODO: refactor to select a random vm of the specified node type, i.e. first put all matching node types into a list and select a random one
    for (Instance instance : instances){
      if(instance.getApplicationInstance().equals(applicationInstanceId)){
        //look for a VM of the nodeType by getting component
        ApplicationComponent applicationComponent = gibbonClient.controller(ApplicationComponent.class).get(instance.getApplicationComponent());

        LifecycleComponent component = gibbonClient.controller(LifecycleComponent.class)
            .get(applicationComponent.getComponent());

        if(component.getName().equals(nodeType.toString())){
          //delete instance
          LOGGER.debug("Found " + nodeType.toString() + " component: " + component.getId() + " of applicationInstance: " + applicationInstanceId);
          boolean deleted = deleteInstance(gibbonClient, instance);
          if(!deleted){
            LOGGER.error("Deleting instance failed!, GibbonAlgorithm is skipping failure further failure injections for this failureSpec!");
            return false;
          }

          LOGGER.debug("deleted instance: " + instance.getId() + " of applicationInstance: " + applicationInstanceId);
          //delete VM
          deleted = deleteVm(gibbonClient, instance);
          if(!deleted){
            LOGGER.error("Deleting VM failed!, GibbonAlgorithm is skipping failure further failure injections for this failureSpec!");
            return false;
          }
          LOGGER.debug("Deleted virtual machine of applicationInstance: " + applicationInstanceId);

          return true;
        }
      }

    }
    LOGGER.error("No matching application instance ID found for: "+ applicationInstanceId);
    return false;

  }

  private static boolean deleteInstance(Client gibbonClient, Instance instance){

    // DELETE INSTANCE
    gibbonClient.controller(Instance.class).delete(instance);
    // wait until instance is removed
    int tries=0; int maxtries=500;
    boolean instanceDeleted=false;
    while(instanceDeleted == false && tries <= maxtries){
      try {
        Thread.sleep(5000); //5s
        tries++;

        List<Instance> is = gibbonClient.controller(Instance.class).getList();
        boolean found=false;
        for(Instance i : is){
          if(i.getId().equals(instance.getId())){
            found=true;
            break;
          }
        }
        if(!found)
          instanceDeleted=true;
      } catch (InterruptedException e) {
        LOGGER.error("Error while checking instance state in Colosseum!", e);
        tries = maxtries;
        return false;
      }
    }

    return instanceDeleted;
  }


  private static boolean deleteVm(Client gibbonClient, Instance instance){

    VirtualMachine virtualMachine = gibbonClient.controller(VirtualMachine.class).get(instance.getVirtualMachine());
    gibbonClient.controller(VirtualMachine.class).delete(virtualMachine);
    // wait until virtualmachine is removed
    int tries=0;
    int maxtries=500;
    boolean vmDeleted=false;
    while(vmDeleted == false && tries <= maxtries){
      try {
        Thread.sleep(5000); //5s
        tries++;

        List<VirtualMachine> vms = gibbonClient.controller(VirtualMachine.class).getList();
        boolean found=false;
        for(VirtualMachine vm : vms){
          if(vm.getId().equals(instance.getVirtualMachine())){
            found=true;
            break;
          }
        }
        if(!found)
          vmDeleted=true;
      } catch (InterruptedException e) {
        LOGGER.error("Error while checking VM state in Colosseum!", e);
        tries = maxtries;
        return false;
      }
    }

    return vmDeleted;
  }

}
