package de.uulm.omi.evaluation.scenario;

import static org.jeasy.flows.engine.WorkFlowEngineBuilder.aNewWorkFlowEngine;
import static org.jeasy.flows.workflow.ParallelFlow.Builder.aNewParallelFlow;
import static org.jeasy.flows.workflow.RepeatFlow.Builder.aNewRepeatFlow;
import static org.jeasy.flows.workflow.SequentialFlow.Builder.aNewSequentialFlow;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.NoiseyNeighbourScenario;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.evaluation.task.CheckDeployment;
import de.uulm.omi.evaluation.task.CheckWorkload;
import de.uulm.omi.evaluation.task.CleanUpResources;
import de.uulm.omi.evaluation.task.CleanUpRun;
import de.uulm.omi.evaluation.task.Delay;
import de.uulm.omi.evaluation.task.DeployDB;
import de.uulm.omi.evaluation.task.FetchYcsbResult;
import de.uulm.omi.evaluation.task.InitEvaluation;
import de.uulm.omi.evaluation.task.YcsbWorkload;
import de.uulm.omi.utils.YcsbWorkloadType;
import org.jeasy.flows.engine.WorkFlowEngine;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.workflow.SequentialFlow;
import org.jeasy.flows.workflow.WorkFlow;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Daniel Seybold on 11.06.2018.
 */
public class NoiseyNeighbour extends EvaluationScenario {

  private final boolean cleanUp;
  private final NoiseyNeighbourScenario noiseyNeighbourScenario;


  private final ClusterContext dbmsCluster;
  private final WorkloadPhaseContext workloadPhaseContext;
  private  final Logger LOGGER = LoggerFactory.getLogger(this.getClass());
  private final int overallRuns;

  public NoiseyNeighbour(String scenarioName, Integer runs,
      NoiseyNeighbourScenario noiseyNeighbourScenario, boolean cleanUp) {
    super("noisey-neighbour", "dummycloud", "couchbase", "vanilla", runs);

    this.noiseyNeighbourScenario = noiseyNeighbourScenario;
    this.cleanUp = cleanUp;

    this.dbmsCluster = new ClusterContext();
    this.workloadPhaseContext = new WorkloadPhaseContext();
    this.overallRuns = runs;

  }

  @Override
  public WorkReport execute() {



    InitEvaluation initEvaluationTask = new InitEvaluation(this.evaluationContext);

    DeployDB deployDB = new DeployDB( this.evaluationContext,
        this.noiseyNeighbourScenario.getDbmsSingle(),
        this.noiseyNeighbourScenario.getDbmsNetwork().getType().name(),
        this.dbmsCluster);

    CheckDeployment checkDeployment = new CheckDeployment(this.evaluationContext, this.dbmsCluster);

    CheckWorkload checkWorkload = new CheckWorkload( this.evaluationContext, this.workloadPhaseContext,null);

    FetchYcsbResult fetchLoadResult = new FetchYcsbResult(this.evaluationContext, this.workloadPhaseContext, YcsbWorkloadType.LOAD,null );

    CleanUpResources cleanUpResources = new CleanUpResources(this.evaluationContext, this.dbmsCluster, this.cleanUp);

    YcsbWorkload loadWorkload = new YcsbWorkload(this.evaluationContext, this.dbmsCluster,
        this.noiseyNeighbourScenario.getLoadWorkload(), this.noiseyNeighbourScenario.getWorkloadNetwork(), this.workloadPhaseContext,
        YcsbWorkloadType.TRANSACTION,null);

    //YcsbWorkload transactionWorkload = new YcsbWorkload(this.evaluationData,"noisey-neighbour", this.scenarioId,
      //  this.noiseyNeighbourScenario.getTransactionWorkload(), this.noiseyNeighbourScenario.getWorkloadNetwork());

    //FetchYcsbResult fetchTransactionResult = new FetchYcsbResult(this.evaluationData,"noisey-neighbour", this.scenarioId, YcsbWorkloadType.TRANSACTION );

    Delay delay = new Delay(this.evaluationContext, this.noiseyNeighbourScenario.getDelay());

    CleanUpRun cleanUpRun = new CleanUpRun(this.evaluationContext, this.dbmsCluster);

    //DeployMolpro deployMolpro = new DeployMolpro(TaskType.MOLPRO, this.molproData,
      //  this.scenarioId, noiseyNeighbourScenario.getMolpro(), this.noiseyNeighbourScenario.getDbmsNetwork().getType().name());

    //CheckDeployment checkMolproDeployment = new CheckDeployment(this.molproData,
     //   this.scenarioId);

   // CleanUpResources cleanUpResourcesMolpro = new CleanUpResources(this.molproData, this.scenarioId, this.cleanUp);

    SequentialFlow runFlow;

    //run with cleanUpResources, i.e. remove DBMS and VM instances
    runFlow = aNewSequentialFlow()
        .execute(deployDB)
        .execute(checkDeployment)
        .execute(loadWorkload)
        .execute(checkWorkload)
        .execute(fetchLoadResult)
        .execute(
            aNewParallelFlow()
                .execute(
                    aNewSequentialFlow()
                        //.execute(transactionWorkload)
                        .execute(checkWorkload)
                        //.execute(fetchTransactionResult)
                        .build(),
                    aNewSequentialFlow()
                        .execute(delay)
                        //.execute(deployMolpro)
                        //.execute(checkMolproDeployment)
                        .build())
                .build()
        )
        .execute(cleanUpResources)
        //.execute(cleanUpResourcesMolpro)
        .execute(cleanUpRun)
        .build();



    WorkFlow sensorStorageWorkFlow = aNewSequentialFlow()
        .execute(initEvaluationTask)
        .then(
            aNewRepeatFlow()
                .repeat(
                    runFlow
                )
                .times(this.overallRuns)
                .build()
        )
        .build();

    WorkFlowEngine workFlowEngine = aNewWorkFlowEngine().build();
    WorkReport workReport = workFlowEngine.run(sensorStorageWorkFlow);

    return  workReport;
  }
}
