package de.uulm.omi.evaluation.scenario;

import static org.jeasy.flows.engine.WorkFlowEngineBuilder.aNewWorkFlowEngine;
import static org.jeasy.flows.workflow.ConditionalFlow.Builder.aNewConditionalFlow;
import static org.jeasy.flows.workflow.RepeatFlow.Builder.aNewRepeatFlow;
import static org.jeasy.flows.workflow.SequentialFlow.Builder.aNewSequentialFlow;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.PerformanceYCSBMultiPhase;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.evaluation.task.CheckDeployment;
import de.uulm.omi.evaluation.task.CheckWorkload;
import de.uulm.omi.evaluation.task.CleanUpFailedResources;
import de.uulm.omi.evaluation.task.CleanUpResources;
import de.uulm.omi.evaluation.task.CleanUpRun;
import de.uulm.omi.evaluation.task.DeployDB;
import de.uulm.omi.evaluation.task.FetchYcsbResult;
import de.uulm.omi.evaluation.task.InitEvaluation;
import de.uulm.omi.evaluation.task.InitRun;
import de.uulm.omi.evaluation.task.Plotting;
import de.uulm.omi.evaluation.task.Plotting.PlottingTask;
import de.uulm.omi.evaluation.task.YcsbWorkload;
import de.uulm.omi.utils.YcsbWorkloadType;
import org.jeasy.flows.engine.WorkFlowEngine;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkReportPredicate;
import org.jeasy.flows.workflow.SequentialFlow;
import org.jeasy.flows.workflow.WorkFlow;

/**
 * Created by Daniel Seybold on 19.02.2019.
 */
public class PerformanceYCSBMultiPhaseScenario extends EvaluationScenario {

  private final PerformanceYCSBMultiPhase performanceYCSBMultiPhase;
  private final boolean cleanUp;
  private final ClusterContext dbmsClusterContext;
  private final WorkloadPhaseContext workloadPhaseContext;
  private final int overallRuns;

  public PerformanceYCSBMultiPhaseScenario(String scenarioType, String cloud, String dbmsType, String dbmsConfig, Integer runs,
      PerformanceYCSBMultiPhase scenarioSpec, boolean cleanUp) {
    super(scenarioType,cloud, dbmsType , dbmsConfig, runs);

    this.cleanUp = cleanUp;
    this.performanceYCSBMultiPhase = scenarioSpec;
    this.dbmsClusterContext = new ClusterContext();
    this.workloadPhaseContext = new WorkloadPhaseContext();
    overallRuns = runs;
  }

  @Override
  public WorkReport execute() {

    InitEvaluation initEvaluationTask = new InitEvaluation(this.evaluationContext);

    InitRun initRun = new InitRun(this.evaluationContext);

    DeployDB deployDB = new DeployDB(this.evaluationContext,
        this.performanceYCSBMultiPhase.getDbmsSingle(),
        this.performanceYCSBMultiPhase.getDbmsNetwork().getType().name(), this.dbmsClusterContext);

    CheckDeployment checkDeployment = new CheckDeployment(this.evaluationContext, this.dbmsClusterContext);

    CheckWorkload checkWorkload = new CheckWorkload(this.evaluationContext, this.workloadPhaseContext,
        performanceYCSBMultiPhase.getWorkloadInstances());




    CleanUpResources cleanUpResources = new CleanUpResources(this.evaluationContext, this.dbmsClusterContext, this.cleanUp);

    YcsbWorkload ycsbWorkloadLoad = new YcsbWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.performanceYCSBMultiPhase.getWorkloadLoad(), this.performanceYCSBMultiPhase.getWorkloadNetwork(), this.workloadPhaseContext,
        YcsbWorkloadType.LOAD, performanceYCSBMultiPhase.getWorkloadInstances());

    FetchYcsbResult fetchYcsbResultLoad = new FetchYcsbResult(this.evaluationContext, this.workloadPhaseContext, YcsbWorkloadType.LOAD,
        performanceYCSBMultiPhase.getWorkloadInstances());

    YcsbWorkload ycsbWorkloadTransaction = new YcsbWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.performanceYCSBMultiPhase.getWorkloadTransaction(), this.performanceYCSBMultiPhase.getWorkloadNetwork(), this.workloadPhaseContext, YcsbWorkloadType.TRANSACTION,
        performanceYCSBMultiPhase.getWorkloadInstances());

    FetchYcsbResult fetchYcsbResultTransaction = new FetchYcsbResult(this.evaluationContext, this.workloadPhaseContext, YcsbWorkloadType.TRANSACTION,
        performanceYCSBMultiPhase.getWorkloadInstances());

    CleanUpRun cleanUpRun = new CleanUpRun(this.evaluationContext, dbmsClusterContext);

    CleanUpFailedResources cleanUpFailedResources = new CleanUpFailedResources(this.evaluationContext, this.dbmsClusterContext, this.cleanUp);

    /**
     * Plot the timeseries of the current run
     */




    Plotting mergeLoadYCSB = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.MERGE_YCSB_LOAD);

    Plotting mergeTransactionYCSB = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.MERGE_YCSB_TRANSACTION);

    Plotting evaluationTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.EVALUATION_METRICS);

    Plotting systemTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.SYSTEM_METRICS);

    Plotting workloadTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.WORKLOAD_METRICS, performanceYCSBMultiPhase
        .getWorkloadInstances());


    SequentialFlow failureAwareFlow;

    failureAwareFlow = aNewSequentialFlow()
        .execute(initRun)
        .execute(deployDB)
        .then(aNewConditionalFlow()
            .execute(checkDeployment)
            .when(WorkReportPredicate.COMPLETED)
            .then(aNewSequentialFlow()
                .execute(ycsbWorkloadLoad)
                .execute(checkWorkload)
                .execute(fetchYcsbResultLoad)
                .execute(mergeLoadYCSB)
                .execute(ycsbWorkloadTransaction)
                .execute(checkWorkload)
                .execute(fetchYcsbResultTransaction)
                .execute(mergeTransactionYCSB)
                .execute(evaluationTimeseries)
                .execute(workloadTimeseries)
                .execute(systemTimeseries)
                .execute(cleanUpResources)
                .execute(cleanUpRun)
                .build())
            .otherwise(
                aNewSequentialFlow()
                    .execute(cleanUpFailedResources)
                    .execute(cleanUpRun)
                    .build()
            )
            .build())
        .build();




    WorkFlow sensorStorageWorkFlow = aNewSequentialFlow()
        .execute(initEvaluationTask)
        .then(
            aNewRepeatFlow()
                .repeat(
                    failureAwareFlow
                )
                .times(this.overallRuns)
                .build()
        )
        .build();

    WorkFlowEngine workFlowEngine = aNewWorkFlowEngine().build();
    WorkReport workReport = workFlowEngine.run(sensorStorageWorkFlow);



    return workReport;


  }
}
