package de.uulm.omi.evaluation.scenario;

import static org.jeasy.flows.engine.WorkFlowEngineBuilder.aNewWorkFlowEngine;
import static org.jeasy.flows.workflow.ConditionalFlow.Builder.aNewConditionalFlow;
import static org.jeasy.flows.workflow.RepeatFlow.Builder.aNewRepeatFlow;
import static org.jeasy.flows.workflow.SequentialFlow.Builder.aNewSequentialFlow;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ScalabilityTPCC;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.evaluation.task.CheckDeployment;
import de.uulm.omi.evaluation.task.CheckWorkload;
import de.uulm.omi.evaluation.task.CleanUpFailedResources;
import de.uulm.omi.evaluation.task.CleanUpResources;
import de.uulm.omi.evaluation.task.CleanUpRun;
import de.uulm.omi.evaluation.task.DeployDB;
import de.uulm.omi.evaluation.task.FetchTpcCResult;
import de.uulm.omi.evaluation.task.InitEvaluation;
import de.uulm.omi.evaluation.task.InitRun;
import de.uulm.omi.evaluation.task.Plotting;
import de.uulm.omi.evaluation.task.Plotting.PlottingTask;
import de.uulm.omi.evaluation.task.TpcCWorkload;
import org.jeasy.flows.engine.WorkFlowEngine;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkReportPredicate;
import org.jeasy.flows.workflow.SequentialFlow;
import org.jeasy.flows.workflow.WorkFlow;

/**
 * Created by Daniel Seybold on 10.09.2018.
 */
public class ScalabilityTPCCScenario extends EvaluationScenario {

  private final ScalabilityTPCC scenarioSpec;
  private final boolean cleanUp;
  private final ClusterContext dbmsClusterContext;
  private final WorkloadPhaseContext workloadPhaseContext;
  private final int overallRuns;

  public ScalabilityTPCCScenario(String scenarioType, String cloudType, String dbmsType, String dbmsConfig, Integer runs,
      ScalabilityTPCC scenarioSpec, boolean cleanUp) {
    super(scenarioType,cloudType,  dbmsType , dbmsConfig, runs);

    this.cleanUp = cleanUp;
    this.scenarioSpec = scenarioSpec;
    this.dbmsClusterContext = new ClusterContext();
    this.workloadPhaseContext = new WorkloadPhaseContext();
    overallRuns = runs;
  }

  @Override
  public WorkReport execute() {

    InitEvaluation initEvaluationTask = new InitEvaluation(this.evaluationContext);

    InitRun initRun = new InitRun(this.evaluationContext);

    DeployDB deployDB = new DeployDB(this.evaluationContext,
        this.scenarioSpec.getDbmsCluster(),
        this.scenarioSpec.getDbmsNetwork().getType().name(), this.dbmsClusterContext);

    CheckDeployment checkDeployment = new CheckDeployment(this.evaluationContext, this.dbmsClusterContext);

    CheckWorkload checkWorkload = new CheckWorkload(this.evaluationContext, this.workloadPhaseContext,null);

    FetchTpcCResult fetchTpcCResult = new FetchTpcCResult(this.evaluationContext);


    CleanUpResources cleanUpResources = new CleanUpResources(this.evaluationContext, this.dbmsClusterContext, this.cleanUp);

    TpcCWorkload tpccWorkload = new TpcCWorkload(this.evaluationContext, this.dbmsClusterContext,
        this.scenarioSpec.getWorkload(), this.scenarioSpec.getWorkloadNetwork(), this.workloadPhaseContext);

    CleanUpRun cleanUpRun = new CleanUpRun(this.evaluationContext, dbmsClusterContext);

    CleanUpFailedResources cleanUpFailedResources = new CleanUpFailedResources(this.evaluationContext, this.dbmsClusterContext, this.cleanUp);

    /**
     * Plot the timeseries of the current run
     */

    Plotting evaluationTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.EVALUATION_METRICS);

    Plotting systemTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.SYSTEM_METRICS);

    Plotting workloadTimeseries = new Plotting(this.evaluationContext, this.dbmsClusterContext, PlottingTask.WORKLOAD_METRICS);


    SequentialFlow failureAwareFlow;

    failureAwareFlow = aNewSequentialFlow()
        .execute(initRun)
        .execute(deployDB)
        .then(aNewConditionalFlow()
            .execute(checkDeployment)
            .when(WorkReportPredicate.COMPLETED)
            .then(aNewSequentialFlow()
                .execute(tpccWorkload)
                .execute(checkWorkload)
                .execute(fetchTpcCResult)
                .execute(evaluationTimeseries)
                .execute(workloadTimeseries)
                .execute(systemTimeseries)
                .execute(cleanUpResources)
                .execute(cleanUpRun)
                .build())
            .otherwise(
                aNewSequentialFlow()
                    .execute(cleanUpFailedResources)
                    .execute(cleanUpRun)
                    .build()
            )
            .build())
        .build();




    WorkFlow sensorStorageWorkFlow = aNewSequentialFlow()
        .execute(initEvaluationTask)
        .then(
            aNewRepeatFlow()
                .repeat(
                    failureAwareFlow
                )
                .times(this.overallRuns)
                .build()
        )
        .build();

    WorkFlowEngine workFlowEngine = aNewWorkFlowEngine().build();
    WorkReport workReport = workFlowEngine.run(sensorStorageWorkFlow);

    return workReport;


  }
}