package de.uulm.omi.evaluation.task;

import static com.google.common.base.Preconditions.checkNotNull;

import de.uulm.omi.cloudiator.catalogue.client.ApiException;
import de.uulm.omi.cloudiator.catalogue.client.api.UtilsApi;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Deployment;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Deployment.DeploymentStateEnum;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.utils.CatalogueClientHelper;
import java.math.BigDecimal;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 07.02.2018.
 */
public class CheckDeployment extends EvaluationTask{

  private final ClusterContext clusterContext;
  public CheckDeployment(EvaluationContext evaluationContext, ClusterContext clusterContext) {
    super(TaskType.CHECK_DEPLOYMENT, evaluationContext);
    this.clusterContext = clusterContext;
  }


  @Override
  public WorkReport call() {


    String loggerPrefix= "RUN " + this.evaluationContext.getRunCounter().intValue() +": ";

    UtilsApi utilsClient = CatalogueClientHelper
        .getUtilsClient();

    Long appInstanceId = null;
    appInstanceId = Long.valueOf(this.clusterContext.getIdApplicationInstance());
    checkNotNull(appInstanceId);

    try {
      Deployment deployment = utilsClient.utilsDeploymentStateGet(BigDecimal.valueOf(appInstanceId));

      while (!deployment.getDeploymentState().equals(DeploymentStateEnum.FINISHED)){
        if(deployment.getDeploymentState().equals(DeploymentStateEnum.FAILED)){
          LOGGER.warn(loggerPrefix + "Deployment failed! Stopping!");
          return new DefaultWorkReport(WorkStatus.FAILED);
        }
        LOGGER.debug(loggerPrefix + "Deployment of applicationInstanceId: " + deployment.getApplicationInstanceId()
            + " still in state: " + deployment.getDeploymentState().getValue() + "...checking again in 20s");
        Thread.sleep(20000);
        deployment = utilsClient.utilsDeploymentStateGet(BigDecimal.valueOf(appInstanceId));

      }
      LOGGER.debug(loggerPrefix + "Deployment successfully finished!");
      return new DefaultWorkReport(WorkStatus.COMPLETED);

    } catch (ApiException e) {
      LOGGER.error(loggerPrefix + "Error while checking deployment state!" , e);
      return new DefaultWorkReport(WorkStatus.FAILED);
    } catch (InterruptedException e) {
      LOGGER.error(loggerPrefix + "Error while checking deployment state, got interrupted!" , e);
      return new DefaultWorkReport(WorkStatus.FAILED);
    }

  }
}
