package de.uulm.omi.evaluation.task;

import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ProcessStatus;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadProcessInstance;
import de.uulm.omi.dbms.workload.api.DefaultApi;
import de.uulm.omi.dbms.workload.client.ApiException;
import de.uulm.omi.utils.WorkloadApiClientHelper;
import java.util.List;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 23.01.2018.
 */
public class CheckWorkload extends EvaluationTask {

  private final WorkloadPhaseContext workloadPhaseContext;
  private final List<String> workloadInstances;

  public CheckWorkload(EvaluationContext evaluationContext, WorkloadPhaseContext workloadPhaseContext, List<String> workloadInstances) {
    super(TaskType.CHECK_WORKLOAD,  evaluationContext);
    this.workloadPhaseContext = workloadPhaseContext;
    this.workloadInstances = workloadInstances;
  }

  @Override
  public WorkReport call() {

    String loggerPrefix= "RUN " + this.evaluationContext.getRunCounter().intValue() +": ";

    List<DefaultApi> apiInstances = WorkloadApiClientHelper.getWorkloadClients(workloadInstances);

    //TODO: could be refactored to have parallel threads to check for the workload states
    //TODO: use guava retry mechanism instead
    for(DefaultApi apiInstance : apiInstances){

      try {
        //TODO: check either here for all worklaods running on multiple worklaod API instancs or handle on workflow level
        WorkloadProcessInstance workloadProcessStatus = apiInstance.workloadStatusGet("dummy");
        while (!workloadProcessStatus.getProcessStatus().equals(ProcessStatus.IDLE)){
          if(workloadProcessStatus.getProcessStatus().equals(ProcessStatus.ERROR)){
            LOGGER.debug(loggerPrefix + "Workload execution resulted in error state!!");
            return new DefaultWorkReport(WorkStatus.FAILED);
          }
          LOGGER.debug(loggerPrefix + "Workload is in state: " + workloadProcessStatus.getProcessStatus().getValue());
          Thread.sleep(20000);
          workloadProcessStatus = apiInstance.workloadStatusGet("dummy");
        }

        LOGGER.debug(loggerPrefix + "Workload execution of instance " + apiInstance.getApiClient().getBasePath() + " finished! Workload-API status: " + workloadProcessStatus.getProcessStatus().getValue());


      } catch (ApiException e) {
        LOGGER.error(loggerPrefix + "Error calling the workload-API!",e);
        return new DefaultWorkReport(WorkStatus.FAILED);
      } catch (InterruptedException e) {
        LOGGER.error(loggerPrefix + "Checking thread got interrupted!",e);
        return new DefaultWorkReport(WorkStatus.FAILED);
      }
    }


    return new DefaultWorkReport(WorkStatus.COMPLETED);




  }
}
