package de.uulm.omi.evaluation.task;

import de.uniulm.omi.cloudiator.colosseum.client.Client;
import de.uniulm.omi.cloudiator.colosseum.client.entities.ApplicationInstance;
import de.uniulm.omi.cloudiator.colosseum.client.entities.Instance;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.utils.ColosseumClientHelper;
import de.uulm.omi.utils.ColosseumUtils;
import java.util.List;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Daniel Seybold on 17.07.2018.
 *
 * CleanUpResources of the failed evaluation run by
 * - deleting all associated resources (VMs, containers,...)
 * - skipping the deletion of component instances as this might result in a non-recoverable state
 */
public class CleanUpFailedResources extends EvaluationTask {

  private final Logger LOGGER = LoggerFactory.getLogger(this.getClass());
  private final boolean trigger;
  private final ClusterContext clusterContext;

  private String loggerPrefix = "";

  public CleanUpFailedResources(EvaluationContext evaluationContext, ClusterContext clusterContext,
      boolean trigger) {
    super(TaskType.CLEANUP, evaluationContext);
    this.trigger = trigger;
    this.clusterContext = clusterContext;
  }

  @Override
  public WorkReport call() {

    this.loggerPrefix = "RUN " + this.evaluationContext.getRunCounter().intValue() + ":";



    try {
      if (trigger) {
        LOGGER.info(loggerPrefix + "################# Evaluataion FAILED, probabely due to deploymenet timeout! Skipping  cleanup of resources as this is currently "
            + "not posible for a FAILED run...!");
        //cleanApplicationInstance();
        LOGGER.info(loggerPrefix + "CleanUpFailedResources finished!");
        return new DefaultWorkReport(WorkStatus.COMPLETED);
      } else {
        LOGGER.info(loggerPrefix + "Skipping cleanup failed run as specified!");
        return new DefaultWorkReport(WorkStatus.COMPLETED);
      }
    } catch (Exception e) {
      LOGGER.error(loggerPrefix + "CleanUpFailedResources failed! No everything is broken :-(", e);
      return new DefaultWorkReport(WorkStatus.FAILED);
    }


  }


  private void cleanApplicationInstance() {
    Client cleanupClient = ColosseumClientHelper.getColosseumClient();

    Long appInstanceId = Long.valueOf(this.clusterContext.getIdApplicationInstance());
    LOGGER.debug(
        loggerPrefix + "Cleaning up all Instances of ApplicationInstanceId:" + appInstanceId);

    ApplicationInstance appInstance = cleanupClient.controller(ApplicationInstance.class)
        .get(appInstanceId);
    if (appInstance == null) {
      throw new IllegalStateException(
          loggerPrefix + "No ApplicationInstance available for ApplicationInstanceId: "
              + appInstanceId);
    }
    //delete instance
    List<Instance> instances = cleanupClient.controller(Instance.class).getList();
    for (Instance instance : instances) {
      if (instance.getApplicationInstance().equals(appInstance.getId())) {
        LOGGER.debug(loggerPrefix + "Cleaning up VM:" + instance.getVirtualMachine() + "of instance: " + instance.getId() );
        //ColosseumUtils.deleteInstance(cleanupClient, instance);
        ColosseumUtils.deleteVM(cleanupClient, instance);

      }
    }

    //delete application instance
    cleanupClient.controller(ApplicationInstance.class).delete(appInstance);

    //TODO: what else to delete, IPAddresses, ApplicationInstance?

  }


}



