package de.uulm.omi.evaluation.task;

import de.uniulm.omi.cloudiator.colosseum.client.Client;
import de.uniulm.omi.cloudiator.colosseum.client.entities.Cloud;
import de.uniulm.omi.cloudiator.colosseum.client.entities.Hardware;
import de.uniulm.omi.cloudiator.colosseum.client.entities.HardwareOffer;
import de.uniulm.omi.cloudiator.colosseum.client.entities.Image;
import de.uniulm.omi.cloudiator.colosseum.client.entities.Location;
import de.uulm.omi.cloudiator.catalogue.client.ApiException;
import de.uulm.omi.cloudiator.catalogue.client.api.DatabaseApi;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.CustomConfigurationItem;
import de.uulm.omi.cloudiator.dbms.evaluator.model.DatabaseCluster;
import de.uulm.omi.cloudiator.dbms.evaluator.model.DatabaseDataComponent;
import de.uulm.omi.cloudiator.dbms.evaluator.model.DatabaseManagementComponent;
import de.uulm.omi.cloudiator.dbms.evaluator.model.DatabaseSeedComponent;
import de.uulm.omi.cloudiator.dbms.evaluator.model.DatabaseSingle;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Deployment;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Deployment.DeploymentStateEnum;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.ResourceMapping;
import de.uulm.omi.utils.CatalogueClientHelper;
import de.uulm.omi.utils.ColosseumClientHelper;
import de.uulm.omi.utils.EntrypointProperties;
import java.io.File;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 23.01.2018.
 */
public class DeployDB extends EvaluationTask {

  private DatabaseCluster databaseCluster;
  private DatabaseSingle databaseSingle;
  private final String dbmsNetwork;
  private final ClusterContext clusterContext;




  public DeployDB(EvaluationContext evaluationContext, DatabaseSingle databaseSingle,String dbmsNetwork, ClusterContext dbmsClusterContext) {
    super(TaskType.DEPLOY_DB, evaluationContext);
    this.databaseSingle = databaseSingle;
    this.dbmsNetwork = dbmsNetwork;
    this.clusterContext = dbmsClusterContext;

    CustomConfigurationItem influxUrlConfigurationItem = new CustomConfigurationItem();
    influxUrlConfigurationItem.setEnvName("INFLUXDB_URL");
    influxUrlConfigurationItem.setEnvValue(this.properties.getProperty(EntrypointProperties.INFLUXDB_URL));
    this.databaseSingle.getDatabaseSeedComponent().getCustomConfiguration().add(influxUrlConfigurationItem);

    CustomConfigurationItem influxDbConfigurationItem = new CustomConfigurationItem();
    influxDbConfigurationItem.setEnvName("INFLUXDB_DB");
    influxDbConfigurationItem.setEnvValue(this.properties.getProperty(EntrypointProperties.INFLUXDB_DATABASE));
    this.databaseSingle.getDatabaseSeedComponent().getCustomConfiguration().add(influxDbConfigurationItem);

    CustomConfigurationItem influxScenarioTagConfigurationItem = new CustomConfigurationItem();
    influxScenarioTagConfigurationItem.setEnvName("SCENARIO_ID");
    influxScenarioTagConfigurationItem.setEnvValue(this.evaluationContext.getScenarioType());
    this.databaseSingle.getDatabaseSeedComponent().getCustomConfiguration().add(influxScenarioTagConfigurationItem);

  }


  public DeployDB(EvaluationContext evaluationContext,
      DatabaseCluster databaseCluster, String dbmsNetwork, ClusterContext dbmsClusterContext) {
    super(TaskType.DEPLOY_DB, evaluationContext);
    this.databaseCluster = databaseCluster;
    this.dbmsNetwork = dbmsNetwork;
    this.clusterContext = dbmsClusterContext;

    for ( DatabaseSeedComponent seedComponent: this.databaseCluster.getDatabaseSeedComponent()) {

      CustomConfigurationItem influxUrlConfigurationItem = new CustomConfigurationItem();
      influxUrlConfigurationItem.setEnvName("INFLUXDB_URL");
      influxUrlConfigurationItem.setEnvValue(this.properties.getProperty(EntrypointProperties.INFLUXDB_URL));
      seedComponent.getCustomConfiguration().add(influxUrlConfigurationItem);

      CustomConfigurationItem influxDbConfigurationItem = new CustomConfigurationItem();
      influxDbConfigurationItem.setEnvName("INFLUXDB_DB");
      influxDbConfigurationItem.setEnvValue(this.properties.getProperty(EntrypointProperties.INFLUXDB_DATABASE));
      seedComponent.getCustomConfiguration().add(influxDbConfigurationItem);

      CustomConfigurationItem influxScenarioTagConfigurationItem = new CustomConfigurationItem();
      influxScenarioTagConfigurationItem.setEnvName("SCENARIO_ID");
      influxScenarioTagConfigurationItem.setEnvValue(this.evaluationContext.getScenarioType());
      seedComponent.getCustomConfiguration().add(influxScenarioTagConfigurationItem);

    }

    for ( DatabaseDataComponent dataComponent: this.databaseCluster.getDatabaseDataComponent()) {

      CustomConfigurationItem influxUrlConfigurationItem = new CustomConfigurationItem();
      influxUrlConfigurationItem.setEnvName("INFLUXDB_URL");
      influxUrlConfigurationItem.setEnvValue(this.properties.getProperty(EntrypointProperties.INFLUXDB_URL));
      dataComponent.getCustomConfiguration().add(influxUrlConfigurationItem);

      CustomConfigurationItem influxDbConfigurationItem = new CustomConfigurationItem();
      influxDbConfigurationItem.setEnvName("INFLUXDB_DB");
      influxDbConfigurationItem.setEnvValue(this.properties.getProperty(EntrypointProperties.INFLUXDB_DATABASE));
      dataComponent.getCustomConfiguration().add(influxDbConfigurationItem);

      CustomConfigurationItem influxScenarioTagConfigurationItem = new CustomConfigurationItem();
      influxScenarioTagConfigurationItem.setEnvName("SCENARIO_ID");
      influxScenarioTagConfigurationItem.setEnvValue(this.evaluationContext.getScenarioType());
      dataComponent.getCustomConfiguration().add(influxScenarioTagConfigurationItem);

    }

    for ( DatabaseManagementComponent managementComponent : this.databaseCluster.getDatabaseManagementComponent()) {

      CustomConfigurationItem influxUrlConfigurationItem = new CustomConfigurationItem();
      influxUrlConfigurationItem.setEnvName("INFLUXDB_URL");
      influxUrlConfigurationItem.setEnvValue(this.properties.getProperty(EntrypointProperties.INFLUXDB_URL));
      managementComponent.getCustomConfiguration().add(influxUrlConfigurationItem);

      CustomConfigurationItem influxDbConfigurationItem = new CustomConfigurationItem();
      influxDbConfigurationItem.setEnvName("INFLUXDB_DB");
      influxDbConfigurationItem.setEnvValue(this.properties.getProperty(EntrypointProperties.INFLUXDB_DATABASE));
      managementComponent.getCustomConfiguration().add(influxDbConfigurationItem);

      CustomConfigurationItem influxScenarioTagConfigurationItem = new CustomConfigurationItem();
      influxScenarioTagConfigurationItem.setEnvName("SCENARIO_ID");
      influxScenarioTagConfigurationItem.setEnvValue(this.evaluationContext.getScenarioType());
      managementComponent.getCustomConfiguration().add(influxScenarioTagConfigurationItem);

    }


  }

  private void logResourceIdMapping(DatabaseSeedComponent databaseSeedComponent){

    ResourceMapping resourceMapping = new ResourceMapping();


    Client client = ColosseumClientHelper.getColosseumClient();



    /**
     * cloud mapping
     */
    Cloud cloud = client.controller(Cloud.class).get(databaseSeedComponent.getResource().getIdCloud().longValue());
    de.uulm.omi.cloudiator.dbms.evaluator.model.Cloud mowgliCloud = new de.uulm.omi.cloudiator.dbms.evaluator.model.Cloud();
    mowgliCloud.setId(databaseSeedComponent.getResource().getIdCloud().longValue());
    mowgliCloud.setName(cloud.getName());
    resourceMapping.setCloud(mowgliCloud);

    /**
     * location mapping
     */
    Location location = client.controller(Location.class).get(databaseSeedComponent.getResource().getIdLocation().longValue());
    de.uulm.omi.cloudiator.dbms.evaluator.model.Location mowgliLocation = new de.uulm.omi.cloudiator.dbms.evaluator.model.Location();
    mowgliLocation.setId(databaseSeedComponent.getResource().getIdLocation().longValue());
    mowgliLocation.setName(location.getName());
    resourceMapping.setLocation(mowgliLocation);

    /**
     * location mapping
     */
    Image image = client.controller(Image.class).get(databaseSeedComponent.getResource().getIdImage().longValue());
    de.uulm.omi.cloudiator.dbms.evaluator.model.Image mowgliImage = new de.uulm.omi.cloudiator.dbms.evaluator.model.Image();
    mowgliImage.setId(databaseSeedComponent.getResource().getIdImage().longValue());
    mowgliImage.setName(image.getName());
    resourceMapping.setImage(mowgliImage);

    /**
     * hardware mapping
     */
    Hardware hardware = client.controller(Hardware.class).get(databaseSeedComponent.getResource().getIdHardware().longValue());
    resourceMapping.setIdHardware(databaseSeedComponent.getResource().getIdHardware().longValue());
    resourceMapping.setHardwareName(hardware.getName());
    HardwareOffer hardwareOffer = client.controller(HardwareOffer.class).get(hardware.getHardwareOffer());
    de.uulm.omi.cloudiator.dbms.evaluator.model.Hardware mowgliHardware = new de.uulm.omi.cloudiator.dbms.evaluator.model.Hardware();
    mowgliHardware.setCores(Long.valueOf(hardwareOffer.getNumberOfCores()));
    mowgliHardware.setMemory(hardwareOffer.getMbOfRam());
    mowgliHardware.setDisk(hardwareOffer.getLocalDiskSpace());
    resourceMapping.setHardwareSpec(mowgliHardware);

    String specUri =  this.evaluationContext.getEvluationRun()
        + File.separator + EvaluationTask.SPECS_FOLDER
        + File.separator + EvaluationTask.RESOURCE_MAPPING_SPEC;

    this.writeReport(specUri, GSON.toJson(resourceMapping));

  }

  @Override
  public WorkReport call() {


    String loggerPrefix= "RUN " + this.evaluationContext.getRunCounter().intValue() +": ";


    try {
      //get catalogue client
      DatabaseApi databaseClient = CatalogueClientHelper
          .getDatabaseClient();

      String specUri =  this.evaluationContext.getEvluationRun()
          + File.separator + EvaluationTask.SPECS_FOLDER
          + File.separator + EvaluationTask.DBMS_SPEC;

      Deployment result;
      if(databaseSingle != null && databaseCluster == null){
        LOGGER.debug( loggerPrefix + "deploying a single DBMS node...");

        this.writeReport(specUri, GSON.toJson(this.databaseSingle));

        this.logResourceIdMapping(this.databaseSingle.getDatabaseSeedComponent());

        result = databaseClient
            .databaseSinglePost(this.dbmsNetwork, databaseSingle);


      }else if(databaseCluster != null && databaseSingle == null){
        LOGGER.debug( loggerPrefix + "deploying a DBMS cluster...");

        this.writeReport(specUri, GSON.toJson(this.databaseCluster));
        this.logResourceIdMapping(this.databaseCluster.getDatabaseSeedComponent().get(0));

        result = databaseClient
            .databaseClusterPost(this.dbmsNetwork, databaseCluster);

      }else{
        LOGGER.error(loggerPrefix + "No DB for deployment provided!");
        return new DefaultWorkReport(WorkStatus.FAILED);
      }


      if(result.getDeploymentState().equals(DeploymentStateEnum.RUNNING)){

        clusterContext.setIdApplicationInstance(
            String.valueOf(result.getApplicationInstanceId()));


        return new DefaultWorkReport(WorkStatus.COMPLETED);
      }else{
        LOGGER.warn(loggerPrefix + "Unexpected deployment state: " + result.getDeploymentState().getValue());
        return new DefaultWorkReport(WorkStatus.FAILED);
      }
    } catch (ApiException e) {
      LOGGER.error( loggerPrefix + "Error while deploying a database cluster!", e);
      return new DefaultWorkReport(WorkStatus.FAILED);

    }
  }
}
