package de.uulm.omi.evaluation.task;

import de.uulm.omi.cloudiator.catalogue.client.ApiException;
import de.uulm.omi.cloudiator.catalogue.client.api.ApplicationApi;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Component;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Deployment;
import de.uulm.omi.cloudiator.dbms.evaluator.model.Deployment.DeploymentStateEnum;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.utils.CatalogueClientHelper;
import java.io.File;
import java.util.List;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 11.06.2018.
 */
public class DeployMolpro extends EvaluationTask {

  private final List<Component> components;
  private final String networkType;


  public DeployMolpro(TaskType taskType,
      EvaluationContext evaluationContext, List<Component> components, String networkType) {
    super(taskType, evaluationContext);

    this.networkType = networkType;
    this.components = components;
  }

  @Override
  public WorkReport call() {
    try {
      //get catalogue client
      ApplicationApi applicationClient = CatalogueClientHelper
          .getApplicationClient();

      String specUri = this.evaluationContext.getEvluationRun() +  File.separator + EvaluationTask.SPECS_FOLDER + File.separator + EvaluationTask.DBMS_SPEC;

      Deployment result;
      LOGGER.debug("deploying a generic application...");

      this.writeReport(specUri, this.components.toString());

      result = applicationClient.genericPost("MOLPRO", networkType, this.components);



      if(result.getDeploymentState().equals(DeploymentStateEnum.RUNNING)){

        //TODO: align with new strucutre
        //EvaluationGroupItem genericItem = new EvaluationGroupItem();
        //genericItem.setApplicationInstanceId(result.getApplicationInstanceId());
        //genericItem.setGroupType(GroupTypeEnum.GENERIC);
        //this.evaluationData.addEvaluationGroupItem(genericItem);


        return new DefaultWorkReport(WorkStatus.COMPLETED);
      }else{
        LOGGER.warn("Unexpected deployment state: " + result.getDeploymentState().getValue());
        return new DefaultWorkReport(WorkStatus.FAILED);
      }
    } catch (ApiException e) {
      LOGGER.error("Error while deploying a generic application!", e);
      return new DefaultWorkReport(WorkStatus.FAILED);

    }
  }
}