package de.uulm.omi.evaluation.task;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.utils.EntrypointProperties;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.Properties;
import org.jeasy.flows.work.Work;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Daniel Seybold on 23.01.2018.
 */
public abstract class EvaluationTask implements Work, ReportService {

  protected   final Logger LOGGER = LoggerFactory.getLogger(this.getClass());

  public static final String SPECS_FOLDER = "specs";
  public static final String LOAD_SPEC = "loadSpec.json";
  public static final String TRANSACTION_SPEC = "transactionSpec.json";
  public static final String DBMS_SPEC = "dbmsSpec.json";
  public static final String RESOURCE_MAPPING_SPEC = "resourceMapping.json";
  public static final String GIBBON_SPEC = "gibbonSpec.json";
  public static final String ELASTICITY_SPEC = "elasticitySpec.json";
  public static final String TASK_LOG = "task.log";

  public static final String DATA_FOLDER = "data";
  public static final String LOAD_FILE = "load.txt";
  public static final String TRANSACTION_FILE = "transaction.txt";

  public static final String TPCC_FILE = "tpcc.txt";

  public static final String PLOTS_FOLDER = "plots";
  public static final String TIMESERIES_LOAD_COLLECTION_FOLDER = "timeseriesLoadData";
  public static final String TIMESERIES_TRANSACTION_COLLECTION_FOLDER = "timeseriesTransactionData";
  public static final String MONITORING_FOLDER = "monitoring";
  public static final String TIMESERIES_FOLDER = "timeseries";
  public static final String TASK_LOG_FOLDER = "taskLogs";

  public static final Gson GSON = new GsonBuilder().setPrettyPrinting().create();

  protected final EvaluationContext evaluationContext;
  protected final TaskType taskType;
  protected final Properties properties;


  public EvaluationTask(TaskType taskType, EvaluationContext evaluationContext){

    this.evaluationContext = evaluationContext;
    this.taskType = taskType;
    this.properties = EntrypointProperties.getInstance().getProperties();

  }

  @Override
  public String getName(){
    return this.taskType.name();
  }


  @Override
  public void writeReport(String uri, String content){

    try {
      Files.write( Paths.get(uri), content.getBytes(), StandardOpenOption.CREATE);
    } catch (IOException e) {
      LOGGER.error("Error while writing workload config!" , e);
    }

  }



}
