package de.uulm.omi.evaluation.task;

import com.google.common.io.Files;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.dbms.workload.api.DefaultApi;
import de.uulm.omi.dbms.workload.client.ApiException;
import de.uulm.omi.utils.WorkloadApiClientHelper;
import java.io.File;
import java.io.IOException;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 10.09.2018.
 */
public class FetchTpcCResult extends EvaluationTask {

  public FetchTpcCResult(EvaluationContext evaluationContext) {
    super(TaskType.FETCH_WORKLOAD, evaluationContext);

  }

  @Override
  public WorkReport call() {

    DefaultApi apiInstance = WorkloadApiClientHelper.getWorkloadClients(null).get(0);

    String loggerPrefix= "RUN " + this.evaluationContext.getRunCounter().intValue() +": ";

    try {
      File tmpResultFile = apiInstance.workloadResultGet("tpcc_" + evaluationContext.getRunCounter().intValue() + "_" + this.evaluationContext.getScenarioType(), "TPCC");
      LOGGER.debug(loggerPrefix + "Tmp result file location:" + tmpResultFile.getAbsolutePath());

      String dataUri = this.evaluationContext.getEvluationRun() +  File.separator + EvaluationTask.DATA_FOLDER + File.separator + EvaluationTask.TPCC_FILE;


      LOGGER.debug(loggerPrefix + "Storing workload result file under: " + dataUri);

      File resultFile = new File(dataUri);

      Files.move(tmpResultFile, resultFile);

    } catch (ApiException e) {
      LOGGER.error(loggerPrefix + "Error while fetching workload result!", e);
      return new DefaultWorkReport(WorkStatus.FAILED);

    } catch (IOException e) {

      LOGGER.error(loggerPrefix + "Error while moving workload result!", e);
      return new DefaultWorkReport(WorkStatus.FAILED);
    }

    LOGGER.debug(loggerPrefix + "Successfully fetched workload result and stored to disk!");
    return new DefaultWorkReport(WorkStatus.COMPLETED);
  }
}