package de.uulm.omi.evaluation.task;

import com.google.common.io.Files;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.dbms.workload.api.DefaultApi;
import de.uulm.omi.dbms.workload.client.ApiException;
import de.uulm.omi.utils.WorkloadApiClientHelper;
import de.uulm.omi.utils.YcsbWorkloadType;
import java.io.File;
import java.io.IOException;
import java.util.List;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 27.04.2018.
 */
public class FetchYcsbResult extends EvaluationTask {

  private final YcsbWorkloadType ycsbWorkloadType;
  private final WorkloadPhaseContext workloadPhaseContext;
  private final List<String> workloadInstances;

  public FetchYcsbResult(EvaluationContext evaluationContext, WorkloadPhaseContext workloadPhaseContext, YcsbWorkloadType ycsbWorkloadType,List<String> workloadInstances) {
    super(TaskType.FETCH_WORKLOAD, evaluationContext);

    this.ycsbWorkloadType = ycsbWorkloadType;
    this.workloadPhaseContext = workloadPhaseContext;
    this.workloadInstances = workloadInstances;

  }

  @Override
  public WorkReport call() {

    List<DefaultApi> apiInstances = WorkloadApiClientHelper.getWorkloadClients(workloadInstances);

    String loggerPrefix= "RUN " + this.evaluationContext.getRunCounter().intValue() +": ";

    for (DefaultApi apiInstance : apiInstances) {

      try {
        File tmpResultFile = apiInstance.workloadResultGet("ycsb_" + evaluationContext.getRunCounter().intValue() + "_" + this.evaluationContext.getScenarioType()+ "_" + ycsbWorkloadType.toString(), "YCSB");
        LOGGER.debug(loggerPrefix + "Tmp result file location:" + tmpResultFile.getAbsolutePath());

        String dataUri;
        if(this.ycsbWorkloadType.equals(YcsbWorkloadType.LOAD)){
          dataUri = this.evaluationContext.getEvluationRun() +  File.separator + EvaluationTask.DATA_FOLDER + File.separator + apiInstances.indexOf(apiInstance) + "_" +  EvaluationTask.LOAD_FILE;

        }else if(this.ycsbWorkloadType.equals(YcsbWorkloadType.TRANSACTION)){
          dataUri = this.evaluationContext.getEvluationRun() +  File.separator + EvaluationTask.DATA_FOLDER + File.separator + apiInstances.indexOf(apiInstance) + "_" + EvaluationTask.TRANSACTION_FILE;
        }else{
          LOGGER.debug(loggerPrefix + "Unsupported Ycsb workload type: " + ycsbWorkloadType.toString());
          return new DefaultWorkReport(WorkStatus.COMPLETED);
        }

        LOGGER.debug(loggerPrefix + "Storing workload result file under: " + dataUri);

        File resultFile = new File(dataUri);

        Files.move(tmpResultFile, resultFile);

      } catch (ApiException e) {
        LOGGER.error(loggerPrefix + "Error while fetching workload result!", e);
        return new DefaultWorkReport(WorkStatus.FAILED);

      } catch (IOException e) {

        LOGGER.error(loggerPrefix + "Error while moving workload result!", e);
        return new DefaultWorkReport(WorkStatus.FAILED);
      }
    }


    LOGGER.debug(loggerPrefix + "Successfully fetched workload result and stored to disk!");
    return new DefaultWorkReport(WorkStatus.COMPLETED);
  }


}
