package de.uulm.omi.evaluation.task;

import static com.google.common.base.Preconditions.checkNotNull;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.GibbonSpec;
import de.uulm.omi.evaluation.jungle.GibbonAlgorithm;
import java.io.File;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 01.03.2018.
 */
public class Gibbon extends EvaluationTask {

  private final GibbonSpec gibbonSpec;
  private final ClusterContext clusterContext;

  public Gibbon(EvaluationContext evaluationContext, ClusterContext dbmsClusterContext, GibbonSpec gibbonSpec) {
    super(TaskType.GIBBON, evaluationContext);

    this.gibbonSpec = gibbonSpec;
    this.clusterContext = dbmsClusterContext;
  }


  @Override
  public WorkReport call() {

    String loggerPrefix= "RUN " + this.evaluationContext.getRunCounter().intValue() +":";


    Long appInstanceId = Long.valueOf(this.clusterContext.getIdApplicationInstance());

    checkNotNull(appInstanceId);
    LOGGER.debug(loggerPrefix + "Executing GibbonAlgorithm for applicationInstanceId: " + appInstanceId);


    String specUri =  this.evaluationContext.getEvluationRun()
        + File.separator + EvaluationTask.SPECS_FOLDER
        + File.separator + EvaluationTask.GIBBON_SPEC;
    this.writeReport(specUri, GSON.toJson(this.gibbonSpec));


    try {
      GibbonAlgorithm.injectFailure(appInstanceId, gibbonSpec);
    }catch (Exception e){
      LOGGER.error(loggerPrefix + "error while executing the GibbonAlgorithm algorithm!" ,e);
      return new DefaultWorkReport(WorkStatus.FAILED);
    }



    return new DefaultWorkReport(WorkStatus.COMPLETED);
  }
}
