package de.uulm.omi.evaluation.task;

import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.utils.EntrypointProperties;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 09.05.2018.
 *
 * This tasks creates the folder structure for the specified scenario
 */
public class InitEvaluation extends EvaluationTask {

  private final int overallRuns;
  private final EvaluationContext evaluationContext;

  private final String loggerPrefix;


  public InitEvaluation(EvaluationContext evaluationContext) {
    super(TaskType.INIT_EVALUATION, evaluationContext);
    this.evaluationContext = evaluationContext;
    this.overallRuns = this.evaluationContext.getRunCounter().intValue();
    this.evaluationContext.setRunCounter(0);
    this.loggerPrefix = this.taskType.name() + ":";
  }

  @Override
  public WorkReport call() {

    //create scenario folder based on scenarioId
    String scenarioDir =
        this.properties.getProperty(EntrypointProperties.SCENARIO_DIR) + File.separator
            + this.evaluationContext.getScenarioType() + File.separator
            + this.evaluationContext.getCloudType() + File.separator
            + this.evaluationContext.getDbmsType() + File.separator + this.evaluationContext
            .getDbmsConfiguration() + File.separator;
    Path scenarioDirPath = Paths.get(scenarioDir);
    //plots folder
    String runPlotsDir =
        this.properties.getProperty(EntrypointProperties.SCENARIO_DIR) + File.separator
            + this.evaluationContext.getScenarioType() + File.separator
            + this.evaluationContext.getCloudType() + File.separator
            + this.evaluationContext.getDbmsType() + File.separator + this.evaluationContext
            .getDbmsConfiguration() + File.separator
            + File.separator + EvaluationTask.PLOTS_FOLDER;
    Path runPlotsPath = Paths.get(runPlotsDir);

    //timeseries collection folder
    String timeseriesLoadCollectionDir = runPlotsDir + File.separator + EvaluationTask.TIMESERIES_LOAD_COLLECTION_FOLDER;
    Path timeseriesLoadDirPath = Paths.get(timeseriesLoadCollectionDir);

    String timeseriesTransactionCollectionDir = runPlotsDir + File.separator + EvaluationTask.TIMESERIES_TRANSACTION_COLLECTION_FOLDER;
    Path timeseriesTransactionCollectionDirPath = Paths.get(timeseriesTransactionCollectionDir);

    try {
      Files.createDirectories(scenarioDirPath);
      Files.createDirectories(runPlotsPath);
      Files.createDirectories(timeseriesLoadDirPath);
      Files.createDirectories(timeseriesTransactionCollectionDirPath);
    } catch (IOException e) {
      LOGGER.error("Error while scenarioDir at: " + scenarioDir, e);
      return new DefaultWorkReport(WorkStatus.FAILED);
    }

    return new DefaultWorkReport(WorkStatus.COMPLETED);

  }
}
