package de.uulm.omi.evaluation.task;

import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.utils.EntrypointProperties;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 13.07.2018.
 */
public class InitRun extends EvaluationTask {

  private String loggerPrefix = "";
  private final EvaluationContext evaluationContext;

  public InitRun(EvaluationContext evaluationContext) {
    super(TaskType.INIT_RUN, evaluationContext);

    this.evaluationContext = evaluationContext;
  }

  @Override
  public WorkReport call() {

    this.loggerPrefix= "RUN " + this.evaluationContext.getRunCounter().intValue() +":";

    this.evaluationContext.setStartTime(System.currentTimeMillis()/1000);

    LOGGER.debug(this.loggerPrefix + " Evaluation run start time: " + this.evaluationContext.getStartTime());

    String evaluationResultFolder = this.properties.getProperty(EntrypointProperties.SCENARIO_DIR) + File.separator + this.evaluationContext.getScenarioType() + File.separator
        + this.evaluationContext.getCloudType() + File.separator
        + this.evaluationContext.getDbmsType() + File.separator + this.evaluationContext.getDbmsConfiguration() + File.separator;

    //create result folder for current evaluation run
    String runDir = evaluationResultFolder + "evaluation_run_" + new SimpleDateFormat("yyyy_MM_dd-HH_mm").format(new Date());

    this.evaluationContext.setEvluationRun(runDir);

      String runDataDir = runDir +  File.separator + EvaluationTask.DATA_FOLDER;
      String runSpecsDir = runDir +  File.separator + EvaluationTask.SPECS_FOLDER;
      String runMonitoringDir = runDir +  File.separator + EvaluationTask.MONITORING_FOLDER;
      String timeseriesDir = runDir +  File.separator + EvaluationTask.TIMESERIES_FOLDER;
      String eventDir = runDir + File.separator + EvaluationTask.TASK_LOG_FOLDER;

      LOGGER.debug(loggerPrefix + "Creating run directory at: " + runDir);

      Path runDirPath = Paths.get(runDir);
      Path runDataPath = Paths.get(runDataDir);
      Path runMonitoringPath = Paths.get(runMonitoringDir);
      Path runSpecsPath = Paths.get(runSpecsDir);
      Path timeseriesPath = Paths.get(timeseriesDir);
      Path eventPath = Paths.get(eventDir);

      try {
        Files.createDirectory(runDirPath);
        Files.createDirectory(runDataPath);
        Files.createDirectory(runSpecsPath);
        Files.createDirectory(runMonitoringPath);
        Files.createDirectory(timeseriesPath);
        Files.createDirectory(eventPath);
      } catch (IOException e) {
        LOGGER.error(loggerPrefix + "Error while creating the " + runDir + "  directory!",e);
        return new DefaultWorkReport(WorkStatus.FAILED);
      }


    String taskLogUri =  this.evaluationContext.getEvluationRun()
        + File.separator + EvaluationTask.TASK_LOG_FOLDER
        + File.separator + EvaluationTask.TASK_LOG;

    String initTaskLogMessage = "AbsoluteTime; RelativeEvaluationTime; TaskType; TaskState\n";
    this.writeReport(taskLogUri, initTaskLogMessage);


    return new DefaultWorkReport(WorkStatus.COMPLETED);
  }
}
