package de.uulm.omi.evaluation.task;

import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 16.10.2018.
 */
public class LogTask extends EvaluationTask {

  private final TaskType taskTypeToLog;
  private final TaskState taskStateToLog;

  public LogTask(EvaluationContext evaluationContext, TaskType taskType, TaskState taskState) {
    super(TaskType.MISC, evaluationContext);

    this.taskTypeToLog = taskType;
    this.taskStateToLog = taskState;


  }

  @Override
  public WorkReport call() {

    long currentTimestamp = System.currentTimeMillis()/1000;

    long eventTimestamp = currentTimestamp - this.evaluationContext.getStartTime();

    String taskLogMessage = new SimpleDateFormat("dd.MM.yyyy-HH:mm:ss").format(new Date()) +"; " +eventTimestamp + "s; " + taskTypeToLog.toString() + "; " + taskStateToLog.toString() + "\n";


    String taskLogUri =  this.evaluationContext.getEvluationRun()
        + File.separator + EvaluationTask.TASK_LOG_FOLDER
        + File.separator + EvaluationTask.TASK_LOG;



    try {
      Files.write( Paths.get(taskLogUri), taskLogMessage.getBytes(), StandardOpenOption.APPEND);
    } catch (IOException e) {
      LOGGER.error("Error while writing TaskLog!" , e);
    }


    return new DefaultWorkReport(WorkStatus.COMPLETED);
  }
}
