package de.uulm.omi.evaluation.task;

import static com.google.common.base.Preconditions.checkNotNull;

import de.uulm.omi.cloudiator.dbms.evaluator.model.ClusterContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.DBEndpoints;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.EvaluationNetwork;
import de.uulm.omi.cloudiator.dbms.evaluator.model.TPCCWorkload;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadPhaseContext;
import de.uulm.omi.cloudiator.dbms.evaluator.model.WorkloadProcessInstance;
import de.uulm.omi.dbms.workload.api.DefaultApi;
import de.uulm.omi.utils.ColosseumUtils;
import de.uulm.omi.utils.DBMSNodeType;
import de.uulm.omi.utils.WorkloadApiClientHelper;
import java.io.File;
import org.jeasy.flows.work.DefaultWorkReport;
import org.jeasy.flows.work.WorkReport;
import org.jeasy.flows.work.WorkStatus;

/**
 * Created by Daniel Seybold on 10.09.2018.
 */
public class TpcCWorkload extends EvaluationTask {

  private final TPCCWorkload tpccWorkload;
  private final EvaluationNetwork network;
  private final ClusterContext clusterContext;
  private final WorkloadPhaseContext workloadPhaseContext;

  private String loggerPrefix = "";

  public TpcCWorkload(EvaluationContext evaluationContext, ClusterContext clusterContext,
      TPCCWorkload tpccWorkload, EvaluationNetwork network, WorkloadPhaseContext workloadPhaseContext) {
    super(TaskType.TPCC, evaluationContext);
    this.tpccWorkload = tpccWorkload;
    this.network = network;
    this.clusterContext = clusterContext;
    this.workloadPhaseContext = workloadPhaseContext;
  }

  @Override
  public WorkReport call() {

    this.loggerPrefix= "RUN " + this.evaluationContext.getRunCounter().intValue() +": ";

    //get workloadApi client
    DefaultApi workloadClient = WorkloadApiClientHelper.getWorkloadClients(null).get(0);

    Long appInstanceId = Long.valueOf(this.clusterContext.getIdApplicationInstance());
    checkNotNull(appInstanceId);

    //build endpoints
    DBEndpoints dbEndpoints = ColosseumUtils.extractDBEndpoints(appInstanceId, this.network.getType().toString(), DBMSNodeType.ALL);
    if(dbEndpoints.isEmpty()){
      LOGGER.warn(loggerPrefix + "Found no endpoints for applicationInstanceId! Aborting!");
      return new DefaultWorkReport(WorkStatus.FAILED);
    }
    tpccWorkload.setDbEndpoints(dbEndpoints);


    //Write Spec to disk
    LOGGER.debug(loggerPrefix + "Running Workload Spec: " + tpccWorkload.toString());
    String specUri = this.evaluationContext.getEvluationRun() +   File.separator + EvaluationTask.SPECS_FOLDER
        + File.separator + EvaluationTask.TRANSACTION_SPEC;
    this.writeReport(specUri, GSON.toJson(tpccWorkload));



    try {
      String workloadTask = "tpcc_" + evaluationContext.getRunCounter().intValue() + "_" + this.evaluationContext.getScenarioType();
      //TODO: pass worklaod API Endpoints from query
      WorkloadProcessInstance result = workloadClient.workloadTpccPost(workloadTask ,tpccWorkload);

      LOGGER.debug(loggerPrefix + "Status: " + result.getProcessStatus().getValue());
      this.workloadPhaseContext.setProcessState(result.getProcessStatus().getValue());

    } catch (de.uulm.omi.dbms.workload.client.ApiException e) {
      LOGGER.error(loggerPrefix + "error calling workload API", e);

      return new DefaultWorkReport(WorkStatus.FAILED);
    }

    return new DefaultWorkReport(WorkStatus.COMPLETED);
  }

}

