package de.uulm.omi.utils;

import static com.google.common.base.Preconditions.checkNotNull;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Daniel Seybold on 23.01.2018.
 */
public enum EntrypointProperties implements EntrypointPropertiesService {


  INSTANCE{

    Properties properties;

    @Override
    public void initProperties(String configPath) {

      checkNotNull(configPath);

      properties = new Properties();
      FileInputStream fileInputStream = null;

      try {

        fileInputStream = new FileInputStream(configPath);
        checkNotNull(fileInputStream);

        // load the properties file
        this.properties.load(fileInputStream);


      } catch (IOException e) {
        LOGGER.error("Error while loading properties file from: " + configPath, e);

      }finally {
        if (fileInputStream != null) {
          try {
            fileInputStream.close();
          } catch (IOException e) {
            LOGGER.error("Error while closing file input stream", e);
          }
        }
      }

    }

    @Override
    public Properties getProperties() {
      return properties;
    }
  };


  //property keys
  public static final String WORKLOAD_API = "workloadApi";
  public static final String APPLICATION_CATALOGUE = "applicationCatalogue";
  public static final String COLOSSEUM = "colosseumUri";
  public static final String INFLUXDB_URL = "influxDbUrl";
  public static final String INFLUXDB_DATABASE = "influxDbDatabase";
  public static final String INFLUXDB_USER = "influxDbUser";
  public static final String INFLUXDB_PASSWORD = "influxDbPassword";
  public static final String SCENARIO_DIR = "scenarioDir";
  public static final String PLOTTING_SCRIPT_DIR = "plottingScriptsDir";


  private  static final Logger LOGGER = LoggerFactory.getLogger(EntrypointProperties.class.getName());

  public static EntrypointProperties getInstance() {
    return EntrypointProperties.INSTANCE;
  }
}
