package de.uulm.omi.utils;

import java.io.File;
import java.io.IOException;
import java.lang.ProcessBuilder.Redirect;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Daniel Seybold on 20.12.2018.
 */
public class ProcessinHelper {

  private static final Logger LOGGER = LoggerFactory.getLogger(ProcessinHelper.class);
  private static final String loggerPrefix = "PROCESSING:";

  public static void plotBoxplots(String data, String labels, String plotName){

    LOGGER.debug(loggerPrefix + "processing boxplots!");

    String plottingScript = EntrypointProperties.getInstance().getProperties().get(EntrypointProperties.PLOTTING_SCRIPT_DIR) + File.separator + "boxplots" + File.separator + "plotService";

    String pattern = "dd-MM-yyyy_HH:mm:ss";
    SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);
    String creationTime = simpleDateFormat.format(new Date());

    String processingLog = EntrypointProperties.getInstance().getProperties().get(EntrypointProperties.SCENARIO_DIR) + File.separator + "boxplot-" + creationTime + ".log";



      ProcessBuilder boxplotProcess = new ProcessBuilder(
          "./plotService",
          data,
          labels,
          EntrypointProperties.getInstance().getProperties().get(EntrypointProperties.SCENARIO_DIR)  + File.separator,
          plotName
         );

      LOGGER.debug(loggerPrefix + "Calling boxplot script: " + boxplotProcess.command().toString());



      try {

        File plottingOutput = new File(processingLog);
        boxplotProcess.redirectErrorStream(true);
        boxplotProcess.redirectOutput(Redirect.appendTo(plottingOutput));

        File workingDir = new File(EntrypointProperties.getInstance().getProperties().get(EntrypointProperties.PLOTTING_SCRIPT_DIR) + File.separator + "boxplots" + File.separator);
        boxplotProcess.directory(workingDir);
        boxplotProcess.start();

      } catch (IOException e) {
        LOGGER.error(loggerPrefix + "Error while calling boxplot service!",e);
      }


  }

}
